@@ -1,5 +1,8 @@
 import { ChatHistoryManager } from './ChatHistoryManager';
-import { ChatMentionDropdown, MentionContext } from './ChatMentionDropdown';
+import {
+  ChatContextMenu,
+  MentionContext
+} from './ChatContextMenu/ChatContextMenu';
 import { Contents } from '@jupyterlab/services';
 import { ToolService } from '../Services/ToolService';
 import { RichTextChatInput } from './RichTextChatInput';
@@ -10,6 +13,7 @@ import { ChatUIHelper } from './ChatUIHelper';
 import { ChatboxContext } from '../Components/ChatboxContext';
 import { ChatRequestStatus } from '../types';
 import { AppStateService } from '../AppState';
+import { NotebookCellStateService } from '../Services/NotebookCellStateService';
 
 /**
  * Input element type that supports both textarea and rich text input
@@ -25,7 +29,7 @@ export class ChatInputManager {
   private userMessageHistory: string[] = [];
   private historyPosition: number = -1;
   private unsavedInput: string = '';
-  private mentionDropdown: ChatMentionDropdown;
+  private mentionDropdown: ChatContextMenu;
 
   // Add map to track all currently active mentions by context.id
   private activeContexts: Map<string, MentionContext> = new Map();
@@ -68,7 +72,7 @@ export class ChatInputManager {
     const inputElement = this.isRichTextInput(chatInput)
       ? chatInput.getInputElement()
       : chatInput;
-    this.mentionDropdown = new ChatMentionDropdown(
+    this.mentionDropdown = new ChatContextMenu(
       inputElement as HTMLElement, // Changed to HTMLElement
       inputContainer,
       contentManager,
@@ -189,11 +193,10 @@ export class ChatInputManager {
     this.userMessageHistory = [];
 
     // Iterate through all notebooks
-    const notebookPaths = this.chatHistoryManager.getNotebookPaths();
-    for (const notebookPath of notebookPaths) {
+    const notebookIds = this.chatHistoryManager.getNotebookIds();
+    for (const notebookId of notebookIds) {
       // Get all threads for this notebook
-      const threads =
-        this.chatHistoryManager.getThreadsForNotebook(notebookPath);
+      const threads = this.chatHistoryManager.getThreadsForNotebook(notebookId);
       if (!threads) continue;
 
       // Extract user messages from each thread
@@ -537,7 +540,7 @@ export class ChatInputManager {
       this.uiHelper.hideLoadingIndicator();
       return;
     }
-
+    const system_messages: string[] = [];
     // Clear the input
     this.clearInput();
 
@@ -549,13 +552,23 @@ export class ChatInputManager {
 
     try {
       // Make sure the conversation service knows which notebook we're targeting
-      const currentNotebookPath =
-        AppStateService.getState().currentNotebookPath;
-      if (currentNotebookPath) {
-        this.conversationService.setNotebookPath(currentNotebookPath);
+      const currentNotebookId = AppStateService.getCurrentNotebookId();
+      if (currentNotebookId) {
+        this.conversationService.setNotebookId(currentNotebookId);
+
+        const cellChanges =
+          await NotebookCellStateService.detectChanges(currentNotebookId);
+        const changesSummary =
+          NotebookCellStateService.generateChangeSummaryMessage(cellChanges);
+        if (changesSummary) {
+          system_messages.push(changesSummary);
+          console.log(
+            '[ChatInputManager] Detected notebook changes, added to system messages'
+          );
+        }
       }
 
-      const system_messages: string[] = [];
+      // Add cell changes to system messages if there are any
 
       const messages = [newUserMessage];
       if (cell_context) {
@@ -575,6 +588,15 @@ export class ChatInputManager {
         system_messages,
         this.modeName
       );
+
+      // Cache the current notebook state after successful message processing
+      if (currentNotebookId)
+        await NotebookCellStateService.cacheCurrentNotebookState(
+          currentNotebookId
+        );
+      console.log(
+        '[ChatInputManager] Cached notebook state after message processing'
+      );
     } catch (error) {
       console.error('Error in conversation processing:', error);
 
@@ -630,10 +652,11 @@ export class ChatInputManager {
 
     if (mentionStart === -1) return;
 
-    // Replace the partial mention with the complete one
+    // Replace the partial mention with the complete one - replace spaces with underscores
     const beforeMention = currentInput.substring(0, mentionStart);
     const afterCursor = currentInput.substring(cursorPos);
-    const replacement = `@${name} `;
+    const displayName = name.replace(/\s+/g, '_');
+    const replacement = `@${displayName} `;
 
     this.setInputValue(beforeMention + replacement + afterCursor);
 
