@@ -7,13 +7,11 @@ import { CodeConfirmationDialog } from '../Components/CodeConfirmationDialog';
 import { NotebookDiffManager } from '../Notebook/NotebookDiffManager';
 import { SettingsWidget } from '../Components/Settings/SettingsWidget';
 import { AppStateService } from '../AppState';
-import {
-  ActionHistory,
-  ActionType,
-  IActionHistoryEntry
-} from './ActionHistory';
+import { ActionHistory, ActionType } from './ActionHistory';
 import { LoadingIndicatorManager } from './ConversationService';
 import { DiffStateService } from '../Services/DiffStateService';
+import { parse } from 'partial-json';
+import { checkTokenLimit } from '../utils/tokenUtils';
 
 export interface ConversationContext {
   chatService: IChatService;
@@ -25,7 +23,7 @@ export interface ConversationContext {
   diffManager: NotebookDiffManager | null;
   actionHistory: ActionHistory;
   autoRun: boolean;
-  notebookPath: string | null;
+  notebookId: string | null;
   templates: Array<{ name: string; content: string }>;
   isActiveToolExecution: boolean;
   chatHistory: HTMLDivElement;
@@ -41,6 +39,7 @@ export interface StreamingState {
     cellId?: string;
     originalContent?: string;
     originalSummary?: string;
+    position?: number;
     summary?: string;
     toolResult?: {
       type: 'tool_result';
@@ -48,20 +47,25 @@ export interface StreamingState {
       content: string;
     };
   };
+  // Queue system for handling streaming operations
+  operationQueue?: {
+    pendingOperation?: NodeJS.Timeout;
+    lastProcessedInput?: string;
+  };
 }
 
 export class ConversationServiceUtils {
   /**
    * Step 1: Initialize conversation processing
    */
-  static initializeConversation(
+  static async initializeConversation(
     context: ConversationContext,
     newMessages: any[],
     systemPromptMessages: string[] = [],
     mode: 'agent' | 'ask' | 'fast' = 'agent'
-  ): { preparedMessages: any[]; tools: any[] } {
+  ): Promise<{ preparedMessages: any[]; tools: any[] }> {
     console.log(
-      `[ConversationService] Processing conversation for notebook: ${context.notebookPath || 'unknown'}`
+      `[ConversationService] Processing conversation for notebook: ${context.notebookId || 'unknown'}`
     );
 
     // Add loading indicator at the start
@@ -103,6 +107,19 @@ export class ConversationServiceUtils {
     streamingState: StreamingState,
     createErrorMessage: (message: any) => Promise<void>
   ): Promise<any> {
+    // Check token count before sending the message
+    const conversationHistory = context.chatService.getConversationHistory();
+    const tokenLimitCheck = checkTokenLimit([
+      ...conversationHistory,
+      ...preparedMessages
+    ]);
+
+    if (tokenLimitCheck.exceeded) {
+      const warningMessage = `💡 **Conversation Length Notice**: This chat has grown quite long (approximately ${tokenLimitCheck.estimatedTokens.toLocaleString()} tokens, ${tokenLimitCheck.percentageUsed}% of the recommended limit). For optimal performance and cost efficiency, consider starting a new chat for your next topic. You can continue with this conversation if needed.`;
+
+      context.messageComponent.addSystemMessage(warningMessage);
+    }
+
     return await context.chatService.sendMessage(
       preparedMessages,
       tools,
@@ -116,11 +133,24 @@ export class ConversationServiceUtils {
           return;
         }
 
-        context.messageComponent.addErrorMessage(
-          `API request failed: ${error.message}. Retrying in 5 seconds... (Attempt ${attempt})`
+        // Progressive backoff delays: 5s, 15s, 30s
+        const retryDelays = [5, 15, 30];
+        const delaySeconds = retryDelays[attempt - 1] || 30;
+
+        // Show user-friendly retry message instead of scary error
+        context.messageComponent.addSystemMessage(
+          `Sage encountered an error, retrying in ${delaySeconds} seconds...`
         );
 
-        context.loadingManager.updateLoadingIndicator('Waiting to retry...');
+        context.loadingManager.updateLoadingIndicator(
+          `Waiting to retry in ${delaySeconds}s...`
+        );
+
+        // Only show the actual error in the chat if this is the final attempt
+        if (attempt >= 3) {
+          // This will be the final attempt, so if it fails, we should show the error
+          // But we don't know yet if it will fail, so we'll handle this in the service
+        }
       },
       // Notebook state fetching function
       () => context.notebookStateService.fetchNotebookState(),
@@ -168,7 +198,7 @@ export class ConversationServiceUtils {
         );
       },
       context.toolService.getContextManager(),
-      context.notebookPath as string,
+      context.notebookId as string,
       createErrorMessage
     );
   }
@@ -182,7 +212,7 @@ export class ConversationServiceUtils {
     streamingState: StreamingState
   ): void {
     if (toolUse.type === 'tool_use_delta') {
-      ConversationServiceUtils.handleToolUseDelta(
+      ConversationServiceUtils.handleToolUseDeltaWithQueue(
         context,
         toolUse,
         streamingState
@@ -194,6 +224,19 @@ export class ConversationServiceUtils {
         streamingState
       );
     } else if (toolUse.type === 'tool_use_stop') {
+      // Process the entire queue immediately when tool use stops
+      if (streamingState.operationQueue?.pendingOperation) {
+        clearTimeout(streamingState.operationQueue.pendingOperation);
+        streamingState.operationQueue.pendingOperation = undefined;
+      }
+
+      // Process any remaining operations in the queue immediately
+      ConversationServiceUtils.processRemainingQueue(
+        context,
+        streamingState,
+        toolUse
+      );
+
       ConversationServiceUtils.handleToolUseStop(
         context,
         toolUse,
@@ -203,9 +246,9 @@ export class ConversationServiceUtils {
   }
 
   /**
-   * Step 3a: Handle tool use delta (streaming input)
+   * Step 3a-queue: Handle tool use delta with queue system to prevent rapid operations
    */
-  static handleToolUseDelta(
+  static handleToolUseDeltaWithQueue(
     context: ConversationContext,
     toolUse: any,
     streamingState: StreamingState
@@ -214,6 +257,11 @@ export class ConversationServiceUtils {
 
     streamingState.streamingToolCall.accumulatedInput += toolUse.input_delta;
 
+    // Initialize queue if not exists
+    if (!streamingState.operationQueue) {
+      streamingState.operationQueue = {};
+    }
+
     // Extract code from partial JSON
     const codeRegex =
       /"(?:source|new_source|updated_plan_string)"\s*:\s*"((?:[^"\\]|\\.)*)/;
@@ -234,24 +282,108 @@ export class ConversationServiceUtils {
     };
 
     if (match && match[1]) {
-      try {
-        const code = JSON.parse(`"${match[1]}"`);
-        ConversationServiceUtils.handleRealTimeNotebookUpdates(
-          context,
-          streamingState.streamingToolCall,
-          code,
-          partialToolUse,
-          toolUse
-        );
+      const code = JSON.parse(`"${match[1]}"`);
 
-        context.messageComponent.updateStreamingToolCall(
-          streamingState.currentStreamingToolCall!,
-          partialToolUse
-        );
-      } catch (e) {
-        // JSON not well-formed yet, ignore and wait for next delta
+      // Clear any pending operations
+      if (streamingState.operationQueue.pendingOperation) {
+        clearTimeout(streamingState.operationQueue.pendingOperation);
+      }
+
+      // Check if this is a significantly different input to avoid unnecessary operations
+      const currentInput = streamingState.streamingToolCall.accumulatedInput;
+      if (streamingState.operationQueue.lastProcessedInput === currentInput) {
+        return; // Skip if input hasn't changed
+      }
+
+      // Queue the operation with a short debounce delay (100ms)
+      streamingState.operationQueue.pendingOperation = setTimeout(() => {
+        try {
+          ConversationServiceUtils.handleRealTimeNotebookUpdates(
+            context,
+            streamingState.streamingToolCall!,
+            code,
+            partialToolUse,
+            toolUse
+          );
+
+          context.messageComponent.updateStreamingToolCall(
+            streamingState.currentStreamingToolCall!,
+            partialToolUse
+          );
+
+          // Mark this input as processed
+          streamingState.operationQueue!.lastProcessedInput = currentInput;
+        } catch (error) {
+          console.error('Error in queued notebook operation:', error);
+        }
+
+        // Clear the pending operation
+        streamingState.operationQueue!.pendingOperation = undefined;
+      }, 100); // 100ms debounce
+    }
+  }
+
+  /**
+   * Step 3a-queue-flush: Process remaining queue items immediately
+   */
+  static processRemainingQueue(
+    context: ConversationContext,
+    streamingState: StreamingState,
+    toolUse: any
+  ): void {
+    if (!streamingState.streamingToolCall || !streamingState.operationQueue) {
+      return;
+    }
+
+    // Extract code from the current accumulated input
+    const codeRegex =
+      /"(?:source|new_source|updated_plan_string)"\s*:\s*"((?:[^"\\]|\\.)*)/;
+    const match =
+      streamingState.streamingToolCall.accumulatedInput.match(codeRegex);
+
+    if (match && match[1]) {
+      const code = JSON.parse(`"${match[1]}"`);
+      const currentInput = streamingState.streamingToolCall.accumulatedInput;
+
+      // Only process if this input hasn't been processed yet
+      if (streamingState.operationQueue.lastProcessedInput !== currentInput) {
+        const partialToolUse = {
+          type: 'tool_use',
+          id: toolUse.id,
+          name: streamingState.streamingToolCall.name,
+          input: {
+            is_streaming: false, // Mark as final processing
+            updated_plan_string: undefined,
+            new_source: undefined,
+            source: undefined,
+            cell_id: undefined
+          }
+        };
+
+        try {
+          ConversationServiceUtils.handleRealTimeNotebookUpdates(
+            context,
+            streamingState.streamingToolCall,
+            code,
+            partialToolUse,
+            toolUse
+          );
+
+          context.messageComponent.updateStreamingToolCall(
+            streamingState.currentStreamingToolCall!,
+            partialToolUse
+          );
+
+          // Mark this input as processed
+          streamingState.operationQueue.lastProcessedInput = currentInput;
+        } catch (error) {
+          console.error('Error in final queue processing:', error);
+        }
       }
     }
+
+    // Clear the queue
+    streamingState.operationQueue = {};
   }
 
   /**
@@ -300,7 +432,7 @@ export class ConversationServiceUtils {
         new_source: toolUse.input.new_source || '',
         summary: toolUse.input.summary || '',
         is_tracking_id: true,
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       });
 
       context.diffManager?.trackEditCell(
@@ -308,7 +440,7 @@ export class ConversationServiceUtils {
         streamingState.streamingToolCall!.originalContent || '',
         toolUse.input.new_source,
         toolUse.input.summary,
-        context.notebookPath
+        context.notebookId
       );
     }
 
@@ -318,13 +450,14 @@ export class ConversationServiceUtils {
         new_source: toolUse.input.source || '',
         summary: toolUse.input.summary || '',
         is_tracking_id: true,
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       });
 
       context.diffManager?.trackAddCell(
         streamingState.streamingToolCall!.cellId!,
         toolUse.input.source,
-        toolUse.input.summary
+        toolUse.input.summary,
+        context.notebookId
       );
     }
 
@@ -333,7 +466,7 @@ export class ConversationServiceUtils {
         updated_plan_string: toolUse.input.updated_plan_string,
         current_step_string: toolUse.input.current_step_string,
         next_step_string: toolUse.input.next_step_string,
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       });
     }
   }
@@ -355,7 +488,7 @@ export class ConversationServiceUtils {
     if (isEditPlan) {
       context.toolService.notebookTools?.stream_edit_plan({
         partial_plan: code,
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       });
       partialToolUse.input.updated_plan_string = code;
     }
@@ -391,6 +524,19 @@ export class ConversationServiceUtils {
     partialToolUse: any,
     toolUse: any
   ): void {
+    console.log('BEGIN HANDLE ADD CELL');
+    console.log(streamingToolCall);
+
+    try {
+      const partialJson = parse(streamingToolCall.accumulatedInput);
+      // Ensure that position
+      if (partialJson.position)
+        streamingToolCall.position = partialJson.position;
+    } catch (error) {
+      console.error('Invalid JSON in accumulated input:', error);
+      throw new Error('Failed to parse accumulated input JSON');
+    }
+
     const cellTypeRegex = /"cell_type"\s*:\s*"([^"]*)"/;
     const cellTypeMatch =
       streamingToolCall.accumulatedInput.match(cellTypeRegex);
@@ -403,35 +549,52 @@ export class ConversationServiceUtils {
       ? cellSummaryMatch[1]
       : 'Cell being created by AI...';
 
-    const validCellTypes = ['code', 'markdown', 'raw'];
+    const validCellTypes = ['code', 'markdown'];
     if (cellType && validCellTypes.includes(cellType)) {
       if (!streamingToolCall.cellId) {
         // First delta: create the cell
-        const newCellId = context.toolService.notebookTools?.add_cell({
-          cell_type: cellType,
-          summary: cellSummary,
-          source: code,
-          notebook_path: context.notebookPath
-        });
+        try {
+          const newCellId = context.toolService.notebookTools?.add_cell({
+            cell_type: cellType,
+            summary: cellSummary,
+            source: code,
+            notebook_path: context.notebookId,
+            position: streamingToolCall.position
+          });
 
-        partialToolUse.input.source = code;
-        partialToolUse.input.cell_id = streamingToolCall.cellId as any;
+          if (!newCellId) {
+            throw new Error('Failed to create new cell, no ID returned');
+          }
 
-        streamingToolCall.cellId = newCellId;
-        streamingToolCall.toolResult = {
-          type: 'tool_result',
-          tool_use_id: toolUse.id,
-          content: newCellId || ''
-        };
+          partialToolUse.input.source = code;
+          partialToolUse.input.cell_id = streamingToolCall.cellId as any;
+
+          streamingToolCall.cellId = newCellId;
+          streamingToolCall.toolResult = {
+            type: 'tool_result',
+            tool_use_id: toolUse.id,
+            content: newCellId || ''
+          };
+
+          console.log(`New cell created with ID: ${newCellId}`);
+        } catch (error) {
+          console.error('Error creating new cell during streaming:', error);
+          throw new Error('Failed to create new cell');
+        }
       } else {
         // Subsequent deltas: edit the cell
-        context.toolService.notebookTools?.edit_cell({
-          cell_id: streamingToolCall.cellId,
-          summary: cellSummary,
-          new_source: code,
-          is_tracking_id: true,
-          notebook_path: context.notebookPath
-        });
+        try {
+          context.toolService.notebookTools?.edit_cell({
+            cell_id: streamingToolCall.cellId,
+            summary: cellSummary,
+            new_source: code,
+            is_tracking_id: true,
+            notebook_path: context.notebookId
+          });
+        } catch (error) {
+          console.error('Error editing existing cell during streaming:', error);
+          throw new Error('Failed to edit existing cell');
+        }
       }
     }
   }
@@ -446,6 +609,7 @@ export class ConversationServiceUtils {
     partialToolUse: any,
     toolUse: any
   ): void {
+    console.log(streamingToolCall);
     if (!streamingToolCall.cellId) {
       const cellIdRegex = /"cell_id"\s*:\s*"([^"]*)"/;
       const cellIdMatch = streamingToolCall.accumulatedInput.match(cellIdRegex);
@@ -457,7 +621,7 @@ export class ConversationServiceUtils {
           try {
             const cellInfo = context.toolService.notebookTools?.findCellByAnyId(
               streamingToolCall.cellId,
-              context.notebookPath
+              context.notebookId
             );
             if (cellInfo) {
               streamingToolCall.originalContent =
@@ -490,7 +654,7 @@ export class ConversationServiceUtils {
         new_source: finalContent,
         summary: streamingToolCall.summary || 'Cell being updated by AI...',
         is_tracking_id: streamingToolCall.cellId.startsWith('cell_'),
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       });
 
       streamingToolCall.toolResult = {
@@ -552,6 +716,15 @@ export class ConversationServiceUtils {
     } else {
       llmStateDisplay?.hide();
     }
+
+    // Also hide DiffNavigationWidget when finalizing tool calls (unless in diff state)
+    if (!llmStateDisplay?.isDiffState()) {
+      const diffNavigationWidget =
+        AppStateService.getDiffNavigationWidgetSafe();
+      if (diffNavigationWidget) {
+        diffNavigationWidget.hidePendingDiffs();
+      }
+    }
   }
 
   /**
@@ -581,12 +754,25 @@ export class ConversationServiceUtils {
       }
     }
 
-    context.messageComponent.addSystemMessage(
-      `cache_input_tokens: ${response.usage.cache_creation_input_tokens}, cache_read_tokens: ${response.usage.cache_read_input_tokens} \n
+    // Check if conversation length has exceeded recommended limits
+    const totalTokens = response.usage.input_tokens || 0;
+    const maxRecommendedTokens = 60000;
+    const tokenLimitExceeded = totalTokens > maxRecommendedTokens;
+
+    let usageMessage = `cache_input_tokens: ${response.usage.cache_creation_input_tokens}, cache_read_tokens: ${response.usage.cache_read_input_tokens} \n
        input_tokens: ${response.usage.input_tokens}, output_tokens: ${response.usage.output_tokens} \n
        user_messages: ${numUserMessages}, assistant_responses: ${numAIResponses} \n
-       tool_calls: ${numToolCalls}, tool_results: ${numToolResults} \n`
-    );
+       tool_calls: ${numToolCalls}, tool_results: ${numToolResults} \n`;
+
+    // Add token limit warning if exceeded
+    if (tokenLimitExceeded) {
+      const percentageUsed = Math.round(
+        (totalTokens / maxRecommendedTokens) * 100
+      );
+      usageMessage += `\n⚠️  **Chat Length Warning**: This conversation has reached ${totalTokens.toLocaleString()} tokens (${percentageUsed}% of recommended limit). For optimal performance and to avoid potential issues, it is recommended to start a new chat session.\n`;
+    }
+
+    context.messageComponent.addSystemMessage(usageMessage);
   }
 
   /**
@@ -685,13 +871,30 @@ export class ConversationServiceUtils {
     const isStreamToolCall =
       streamingState.streamingToolCall &&
       streamingState.streamingToolCall.toolResult;
-    const toolResult = isStreamToolCall
-      ? streamingState.streamingToolCall!.toolResult
-      : await ConversationServiceUtils.processToolCall(context, {
-          id: content.id,
-          name: toolName,
-          input: toolArgs
-        });
+
+    let toolResult = null;
+    if (isStreamToolCall) {
+      toolResult = streamingState.streamingToolCall!.toolResult;
+    } else {
+      const executedApprovedCells =
+        await ConversationServiceUtils.checkExecutedCells(
+          context,
+          content.id,
+          content.input?.cell_id,
+          true
+        );
+
+      if (executedApprovedCells && toolName === 'notebook-run_cell') {
+        console.log('Skipping tool call in chat');
+        return true;
+      }
+
+      toolResult = await ConversationServiceUtils.processToolCall(context, {
+        id: content.id,
+        name: toolName,
+        input: toolArgs
+      });
+    }
 
     context.isActiveToolExecution = false;
 
@@ -843,7 +1046,7 @@ export class ConversationServiceUtils {
           'User rejected changes, cell was not added or executed.',
           {
             cell_id: content.input?.cell_id,
-            notebook_path: context.notebookPath
+            notebook_path: context.notebookId
           }
         );
       }
@@ -881,7 +1084,7 @@ export class ConversationServiceUtils {
         'notebook-run_cell',
         content.id,
         toolResult.content,
-        { cell_id: cellId, notebook_path: context.notebookPath }
+        { cell_id: cellId, notebook_path: context.notebookId }
       );
 
       // Add to conversation history
@@ -892,7 +1095,7 @@ export class ConversationServiceUtils {
             type: 'tool_use',
             id: content.id,
             name: 'notebook-run_cell',
-            input: { cell_id: cellId, notebook_path: context.notebookPath }
+            input: { cell_id: cellId, notebook_path: context.notebookId }
           }
         ]
       };
@@ -989,7 +1192,7 @@ export class ConversationServiceUtils {
       ...toolCall,
       input: {
         ...toolCall.input,
-        notebook_path: context.notebookPath
+        notebook_path: context.notebookId
       }
     };
 
@@ -1037,7 +1240,8 @@ export class ConversationServiceUtils {
             context.diffManager.trackRemoveCell(
               cell.trackingId || cell.id,
               cell.content || '',
-              cell.custom?.summary || 'Removed by AI'
+              cell.custom?.summary || 'Removed by AI',
+              context.notebookId
             );
           }
         }
@@ -1049,20 +1253,22 @@ export class ConversationServiceUtils {
    * Step 7: Handle pending diffs if no tool calls were made
    */
   static async handlePendingDiffsAfterToolCalls(
-    context: ConversationContext,
-    hasToolCalls: boolean
+    context: ConversationContext
   ): Promise<boolean> {
-    if (
-      hasToolCalls &&
-      context.diffManager &&
-      context.diffManager.hasPendingDiffs()
-    ) {
+    if (context.diffManager && context.diffManager.hasPendingDiffs()) {
       context.isActiveToolExecution = true;
 
       const llmStateDisplay =
         AppStateService.getChatContainerSafe()?.chatWidget.llmStateDisplay;
       if (llmStateDisplay) {
-        llmStateDisplay.showPendingDiffs(context.notebookPath);
+        llmStateDisplay.showPendingDiffs(context.notebookId);
+      }
+
+      // Also show diffs in DiffNavigationWidget for synchronized display
+      const diffNavigationWidget =
+        AppStateService.getDiffNavigationWidgetSafe();
+      if (diffNavigationWidget) {
+        diffNavigationWidget.showPendingDiffs(context.notebookId);
       }
 
       await ConversationServiceUtils.checkPendingDiffs(context);
@@ -1113,14 +1319,13 @@ export class ConversationServiceUtils {
       context.chatHistory,
       true,
       false,
-      context.notebookPath || undefined
+      AppStateService.getState().currentNotebookId || undefined
     );
 
     // Wait for the signal to indicate processing is complete
     const approvalStatus =
       await context.diffManager.waitForDiffProcessingComplete();
 
-    context.messageComponent.scrollToBottom();
     return approvalStatus !== DiffApprovalStatus.REJECTED;
   }
 
@@ -1172,14 +1377,13 @@ export class ConversationServiceUtils {
       context.chatHistory,
       true,
       true,
-      context.notebookPath || undefined
+      AppStateService.getState().currentNotebookId || undefined
     );
 
     // Wait for the signal to indicate processing is complete
     const approvalStatus =
       await context.diffManager.waitForDiffProcessingComplete();
 
-    context.messageComponent.scrollToBottom();
     return approvalStatus !== DiffApprovalStatus.REJECTED;
   }
 
@@ -1191,7 +1395,8 @@ export class ConversationServiceUtils {
   static async checkExecutedCells(
     context: ConversationContext,
     contentId: string,
-    requestedCellId?: string
+    requestedCellId?: string,
+    log?: boolean
   ): Promise<boolean> {
     console.log('CHECKING EXECUTED CELLS!');
 
@@ -1199,7 +1404,7 @@ export class ConversationServiceUtils {
 
     // Get all approved cells that have been executed during diff approval
     const approvedCellIds = context.diffManager.getApprovedCellIds(
-      context.notebookPath
+      context.notebookId
     );
     if (approvedCellIds.length === 0) {
       return false;
@@ -1248,7 +1453,7 @@ export class ConversationServiceUtils {
         'notebook-run_cell',
         contentId,
         toolResult.content,
-        { cell_id: requestedCellId, notebook_path: context.notebookPath }
+        { cell_id: requestedCellId, notebook_path: context.notebookId }
       );
 
       return false;
@@ -1261,11 +1466,12 @@ export class ConversationServiceUtils {
         ? executedCells.filter(cell => cell.cellId === requestedCellId)
         : executedCells;
 
-    context.messageComponent.addSystemMessage(
-      requestedCellId && requestedCellWasExecuted
-        ? `Cell ${requestedCellId} was executed during diff approval.`
-        : `${cellsToReport.length} cells were executed during diff approval.`
-    );
+    log &&
+      context.messageComponent.addSystemMessage(
+        requestedCellId && requestedCellWasExecuted
+          ? `Cell ${requestedCellId} was executed during diff approval.`
+          : `${cellsToReport.length} cells were executed during diff approval.`
+      );
 
     // For each executed cell, add the tool result to the conversation
     for (const { cellId, result } of cellsToReport) {
@@ -1281,7 +1487,7 @@ export class ConversationServiceUtils {
         'notebook-run_cell',
         contentId,
         toolResult.content,
-        { cell_id: cellId, notebook_path: context.notebookPath }
+        { cell_id: cellId, notebook_path: context.notebookId }
       );
 
       // Add to conversation history
@@ -1292,7 +1498,7 @@ export class ConversationServiceUtils {
             type: 'tool_use',
             id: contentId,
             name: 'notebook-run_cell',
-            input: { cell_id: cellId, notebook_path: context.notebookPath }
+            input: { cell_id: cellId, notebook_path: context.notebookId }
           }
         ]
       };
@@ -1337,6 +1543,12 @@ export class ConversationServiceUtils {
     context: ConversationContext,
     streamingState: StreamingState
   ): void {
+    // Clear any pending queue operations
+    if (streamingState.operationQueue?.pendingOperation) {
+      clearTimeout(streamingState.operationQueue.pendingOperation);
+      streamingState.operationQueue.pendingOperation = undefined;
+    }
+
     if (streamingState.currentStreamingMessage) {
       context.messageComponent.removeElement(
         streamingState.currentStreamingMessage
