@@ -1,7 +1,6 @@
 import { ChatMessages } from '../Chat/ChatMessages';
 import { ConversationService } from '../Chat/ConversationService';
 import { PanelLayout, Widget } from '@lumino/widgets';
-import { Message } from '@lumino/messaging';
 import { ConfigService } from '../Config/ConfigService';
 import { IChatService } from '../Services/IChatService';
 import { ServiceFactory, ServiceProvider } from '../Services/ServiceFactory';
@@ -16,21 +15,21 @@ import { ChatboxContext } from './ChatboxContext';
 import { NewChatDisplayWidget } from './NewChatDisplayWidget';
 import { LLMStateDisplay } from './LLMStateDisplay/LLMStateDisplay';
 import {
-  HANDS_ON_MODE_ICON,
-  ASK_ICON,
   AGENT_MODE_ICON,
   AGENT_MODE_SHINY_ICON,
+  ASK_ICON,
+  HANDS_ON_MODE_ICON,
   OPEN_MODE_SELECTOR_ICON,
   SEND_ICON
 } from './icons';
 import { PlanStateDisplay } from './PlanStateDisplay';
-import { MoreOptionsDisplay, IMoreOptionsActions } from './MoreOptionsDisplay';
+import { MoreOptionsDisplay } from './MoreOptionsDisplay';
 import { UpdateBannerWidget } from './UpdateBanner/UpdateBannerWidget';
 import { Subscription } from 'rxjs';
 
 // Recommended prompts for new chat display
 const RECOMMENDED_PROMPTS: string[] = [
-  // 'Analyze the data in my notebook',
+  // 'Analyze the data in my notebook'
   // 'Create a visualization from my data',
   // 'Help me clean and preprocess this dataset',
   // 'Build a machine learning model',
@@ -47,6 +46,7 @@ export class ChatBoxWidget extends Widget {
   private newChatButton: HTMLButtonElement;
   private undoButton: HTMLButtonElement;
   public autorunCheckbox: HTMLInputElement;
+  private lastNotebookId: string | null = null;
 
   private threadSelectorButton: HTMLButtonElement;
   private modeSelector: HTMLDivElement;
@@ -62,6 +62,7 @@ export class ChatBoxWidget extends Widget {
   private planStateDisplay: PlanStateDisplay;
   private moreOptionsDisplay: MoreOptionsDisplay;
   private updateBanner: UpdateBannerWidget | null = null;
+  private scrollDownButton: HTMLButtonElement;
 
   // Chat services
   private messageComponent: ChatMessages;
@@ -206,8 +207,55 @@ export class ChatBoxWidget extends Widget {
     historyContainer.className = 'sage-ai-history-container';
     this.chatHistory = document.createElement('div');
     this.chatHistory.className = 'sage-ai-chat-history';
+    this.chatHistory.setAttribute('data-is-scrolled-to-bottom', 'true');
     historyContainer.appendChild(this.chatHistory);
 
+    this.scrollDownButton = document.createElement('button');
+    this.scrollDownButton.className = 'sage-ai-scroll-down-button hidden';
+    this.scrollDownButton.innerHTML = `
+      <svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg">
+        <path d="M8 10.25L12.5 5.75L13.25 6.5L8 11.75L2.75 6.5L3.5 5.75L8 10.25Z" fill="var(--jp-ui-font-color1" />
+      </svg>
+    `;
+    this.scrollDownButton.addEventListener('click', () => {
+      this.messageComponent.scrollToBottom();
+      this.hideScrollDownButton();
+    });
+    historyContainer.appendChild(this.scrollDownButton);
+
+    const chatHistoryResizeObserver = new ResizeObserver(() =>
+      this.handleChatHistoryResize()
+    );
+    chatHistoryResizeObserver.observe(this.chatHistory);
+
+    let userScrolled = false;
+    let userScrollTimeout: NodeJS.Timeout | null = null;
+
+    // Mark user-initiated scroll
+    ['wheel', 'touchstart', 'keydown'].forEach(eventType => {
+      window.addEventListener(eventType, () => {
+        userScrolled = true;
+
+        userScrollTimeout && clearTimeout(userScrollTimeout);
+        userScrollTimeout = setTimeout(() => {
+          userScrolled = false;
+        }, 1000); // reset after 1s
+      });
+    });
+    this.chatHistory.addEventListener('scroll', () => {
+      // As the chat-history height change, we need to check if the user is scrolling
+      // or the chat-history is being resized
+      if (userScrolled) {
+        const isScrolledToBottom = this.updateScrollAttribute();
+
+        if (isScrolledToBottom) {
+          this.hideScrollDownButton();
+        } else {
+          this.showScrollDownButton();
+        }
+      }
+    });
+
     // Create the initial history widget
     this.historyWidget = new Widget({ node: historyContainer });
 
@@ -226,7 +274,8 @@ export class ChatBoxWidget extends Widget {
     this.messageComponent = new ChatMessages(
       this.chatHistory,
       this.chatHistoryManager,
-      AppStateService.getNotebookTools()
+      AppStateService.getNotebookTools(),
+      () => this.handleDisplayScrollDownButton()
     );
 
     // Create input container with text input and send button
@@ -529,7 +578,7 @@ export class ChatBoxWidget extends Widget {
 
           // Add LLM state height if visible
           if (!isLLMHidden && this.llmStateDisplay.node.offsetHeight > 0) {
-            totalStateHeight += this.llmStateDisplay.node.offsetHeight;
+            totalStateHeight += this.llmStateDisplay.node.offsetHeight - 10;
           }
 
           // Add plan state height if visible
@@ -537,11 +586,27 @@ export class ChatBoxWidget extends Widget {
             this.planStateDisplay.getIsVisible() &&
             this.planStateDisplay.node.offsetHeight > 0
           ) {
-            totalStateHeight += this.planStateDisplay.node.offsetHeight;
+            totalStateHeight += this.planStateDisplay.node.offsetHeight - 20;
           }
 
-          // Apply the dynamic padding to history container
-          this.chatHistory.parentElement!.style.paddingBottom = `${totalStateHeight}px`;
+          if (totalStateHeight > 0) {
+            totalStateHeight += 10;
+          }
+
+          const historyContainer = this.chatHistory.parentElement;
+          if (historyContainer) {
+            historyContainer.style.paddingBottom = `${totalStateHeight}px`;
+          }
+
+          const scrollDownButton = <HTMLButtonElement>(
+            historyContainer?.querySelector('.sage-ai-scroll-down-button')
+          );
+          if (scrollDownButton) {
+            const bottom = totalStateHeight ? totalStateHeight - 6 : 0;
+            scrollDownButton.style.bottom = `${bottom}px`;
+          }
+
+          this.handleDisplayScrollDownButton();
         }, 200); // Additional delay to ensure transitions/animations complete
       });
     };
@@ -648,9 +713,7 @@ export class ChatBoxWidget extends Widget {
       AppStateService.getContentManager(),
       this.messageComponent,
       this.chatHistory,
-      this,
-      undefined,
-      this.autorunCheckbox
+      this
     );
 
     // Set the diff manager in the conversation service if available
@@ -689,7 +752,12 @@ export class ChatBoxWidget extends Widget {
 
     const waitingUserReplyBoxManager =
       AppStateService.getWaitingUserReplyBoxManager();
-    waitingUserReplyBoxManager.initialize(inputContainer);
+    waitingUserReplyBoxManager.initialize(this.chatHistory);
+    
+    // Set up the continue callback to send "Continue" message
+    waitingUserReplyBoxManager.setContinueCallback(() => {
+      this.sendContinueMessage();
+    });
 
     // Initialize context display after everything is set up
     this.contextHandler.updateContextDisplay();
@@ -698,6 +766,54 @@ export class ChatBoxWidget extends Widget {
       this.showNewChatDisplay();
   }
 
+  private handleDisplayScrollDownButton(): void {
+    if (this.isScrolledToBottom()) {
+      this.hideScrollDownButton();
+    } else {
+      this.showScrollDownButton();
+    }
+  }
+
+  private showScrollDownButton(): void {
+    this.scrollDownButton.classList.remove('hidden');
+  }
+
+  private hideScrollDownButton(): void {
+    this.scrollDownButton.classList.add('hidden');
+  }
+
+  private handleChatHistoryResize(): void {
+    if (this.isScrolledToBottom()) {
+      this.scrollChatHistoryToBottom();
+    } else {
+      this.handleDisplayScrollDownButton();
+    }
+  }
+
+  public scrollChatHistoryToBottom(): void {
+    this.chatHistory.scrollTop = this.chatHistory.scrollHeight;
+  }
+
+  private updateScrollAttribute(): boolean {
+    const scrollTop = this.chatHistory.scrollTop;
+    const scrollHeight = this.chatHistory.scrollHeight;
+    const isScrolledToBottom =
+      Math.ceil(scrollTop + this.chatHistory.clientHeight) >= scrollHeight;
+
+    this.chatHistory.setAttribute(
+      'data-is-scrolled-to-bottom',
+      isScrolledToBottom.toString()
+    );
+
+    return isScrolledToBottom;
+  }
+
+  private isScrolledToBottom(): boolean {
+    return (
+      this.chatHistory.getAttribute('data-is-scrolled-to-bottom') === 'true'
+    );
+  }
+
   /**
    * Create a custom option element for the mode selector dropdown
    * @param value The value of the option
@@ -886,10 +1002,15 @@ export class ChatBoxWidget extends Widget {
     this.messageComponent.addSystemMessage(`Mode switched to: ${displayName}`);
   }
 
+  public updateNotebookId(newId: string): void {
+    AppStateService.setCurrentNotebookId(newId);
+    this.threadManager.updateNotebookId(newId);
+    this.conversationService.updateNotebookId(newId);
+  }
+
+  // Backward compatibility method
   public updateNotebookPath(newPath: string): void {
-    AppStateService.setCurrentNotebookPath(newPath);
-    this.threadManager.updateNotebookPath(newPath);
-    this.conversationService.updateNotebookPath(newPath);
+    this.updateNotebookId(newPath);
   }
 
   /**
@@ -957,33 +1078,32 @@ export class ChatBoxWidget extends Widget {
   }
 
   /**
-   * Update the notebook path and load its chat history
-   * @param notebookPath Path to the notebook
+   * Update the notebook ID and load its chat history
+   * @param notebookId ID of the notebook
    */
-  public setNotebookPath(notebookPath: string | undefined): void {
-    if (!notebookPath) {
-      AppStateService.setCurrentNotebookPath(null);
-      this.threadManager.setNotebookPath(null);
+  public async setNotebookId(notebookId: string | undefined): Promise<void> {
+    if (!notebookId) {
+      AppStateService.setCurrentNotebookId(null);
+      this.threadManager.setNotebookId(null);
       return;
     }
 
-    const currentNotebookPath = AppStateService.getState().currentNotebookPath;
-    if (notebookPath !== currentNotebookPath) {
-      console.log(`[ChatBoxWidget] Switching to notebook: ${notebookPath}`);
-      AppStateService.setCurrentNotebookPath(notebookPath);
+    if (this.lastNotebookId === notebookId) return;
 
-      // Load the chat history for this notebook
-      this.chatHistoryManager.setCurrentNotebook(notebookPath);
+    this.lastNotebookId = notebookId;
 
-      // Update the thread manager with the current notebook path
-      this.threadManager.setNotebookPath(notebookPath);
+    AppStateService.setCurrentNotebookId(notebookId);
 
-      // Show appropriate widget based on whether there are messages
-      this.showNewChatDisplayOrHistory();
+    // Update the thread manager with the current notebook ID
+    this.threadManager.setNotebookId(notebookId);
 
-      // Update conversation service with the current notebook path
-      this.conversationService.setNotebookPath(notebookPath);
-    }
+    // Show appropriate widget based on whether there are messages
+    await this.showNewChatDisplayOrHistory();
+
+    // Update conversation service with the current notebook ID
+    this.conversationService.setNotebookId(notebookId);
+
+    await this.restoreLastThreadForNotebook(notebookId);
 
     // Refresh user message history when switching notebooks
     this.inputManager.loadUserMessageHistory();
@@ -991,11 +1111,50 @@ export class ChatBoxWidget extends Widget {
     // Update context cells indicator when switching notebooks
     const contextManager = AppStateService.getState().notebookContextManager;
     if (contextManager) {
-      const contextCells = contextManager.getContextCells(notebookPath);
+      const contextCells = contextManager.getContextCells(notebookId);
       this.contextHandler.updateContextCellsIndicator(contextCells.length);
     }
   }
 
+  /**
+   * Try to restore the last selected thread for a notebook
+   * @param notebookId ID of the notebook
+   */
+  private async restoreLastThreadForNotebook(
+    notebookId: string
+  ): Promise<void> {
+    try {
+      // Get the last valid thread for this notebook
+      const lastThread =
+        await this.threadManager.getLastValidThreadForNotebook(notebookId);
+
+      if (lastThread && lastThread.messages.length > 0) {
+        // Found a valid last thread, load it
+        console.log(
+          `[ChatBoxWidget] Restoring last thread: ${lastThread.name} for notebook ${notebookId}`
+        );
+
+        await this.showHistoryWidgetFromThread(lastThread);
+        return;
+      } else {
+        // No valid last thread found, show new chat or history based on available content
+        console.log(
+          `[ChatBoxWidget] No valid last thread found for notebook ${notebookId}, showing default view`
+        );
+        await this.createNewChat();
+        this.showNewChatDisplay();
+        return;
+      }
+    } catch (error) {
+      console.warn(
+        `[ChatBoxWidget] Failed to restore last thread for notebook ${notebookId}:`,
+        error
+      );
+      // Fallback to default behavior
+      await this.showNewChatDisplayOrHistory();
+    }
+  }
+
   /**
    * Create a new chat thread
    */
@@ -1004,8 +1163,8 @@ export class ChatBoxWidget extends Widget {
     AppStateService.getWaitingUserReplyBoxManager().hide();
 
     // Only proceed if we have an active notebook
-    const currentNotebookPath = AppStateService.getState().currentNotebookPath;
-    if (!currentNotebookPath) {
+    const currentNotebookId = AppStateService.getCurrentNotebookId();
+    if (!currentNotebookId) {
       this.messageComponent.addSystemMessage('Please open a notebook first.');
       return;
     }
@@ -1019,7 +1178,7 @@ export class ChatBoxWidget extends Widget {
     }
 
     // Create a new thread
-    const newThread = this.threadManager.createNewThread();
+    const newThread = await this.threadManager.createNewThread();
 
     if (newThread) {
       // Clear action history
@@ -1030,6 +1189,13 @@ export class ChatBoxWidget extends Widget {
       // Switch to new chat display since there are no messages
       this.showNewChatDisplay();
       this.llmStateDisplay.hide();
+
+      // Also hide DiffNavigationWidget when creating new chat
+      const diffNavigationWidget =
+        AppStateService.getDiffNavigationWidgetSafe();
+      if (diffNavigationWidget) {
+        diffNavigationWidget.hidePendingDiffs();
+      }
     }
   }
 
@@ -1077,13 +1243,12 @@ export class ChatBoxWidget extends Widget {
    * Cancel the current message processing
    */
   public cancelMessage(): void {
-    console.log('Cancelling message...');
-    console.log(this.inputManager.getIsProcessingMessage());
     if (!this.inputManager.getIsProcessingMessage()) {
       return;
     }
 
-    console.log('Cancelling message processing...');
+    console.log('Cancelling message...');
+    console.log(this.inputManager.getIsProcessingMessage());
 
     // Cancel the request in the chatService
     this.chatService.cancelRequest();
@@ -1094,7 +1259,7 @@ export class ChatBoxWidget extends Widget {
     // Remove loading indicator
     this.uiHelper.removeLoadingIndicator();
 
-    this.messageComponent.addSystemMessage('Request cancelled by user.');
+    // this.messageComponent.addSystemMessage('Request cancelled by user.');
     this.messageComponent.removeLoadingText();
     this.uiHelper.updateSendButton(this.sendButton, false);
     this.sendButton.classList.add('disabled');
@@ -1115,18 +1280,23 @@ export class ChatBoxWidget extends Widget {
     ) {
       // Show pending diffs in LLMStateDisplay
       if (this.llmStateDisplay) {
-        const currentNotebookPath =
-          AppStateService.getState().currentNotebookPath;
-        this.llmStateDisplay.showPendingDiffs(currentNotebookPath);
+        const currentNotebookId = AppStateService.getCurrentNotebookId();
+        this.llmStateDisplay.showPendingDiffs(currentNotebookId);
+
+        // Also show diffs in DiffNavigationWidget for synchronized display
+        const diffNavigationWidget =
+          AppStateService.getDiffNavigationWidgetSafe();
+        if (diffNavigationWidget) {
+          diffNavigationWidget.showPendingDiffs(currentNotebookId);
+        }
       }
 
       // Use setTimeout to ensure UI updates before showing the dialog
       setTimeout(async () => {
-        const currentNotebookPath =
-          AppStateService.getState().currentNotebookPath;
+        const currentNotebookId = AppStateService.getCurrentNotebookId();
         await diffManager?.showCancellationApprovalDialog(
           this.chatHistory,
-          currentNotebookPath // Pass the notebook path
+          currentNotebookId // Pass the notebook ID
         );
       }, 100);
     } else {
@@ -1135,27 +1305,6 @@ export class ChatBoxWidget extends Widget {
     }
   }
 
-  /**
-   * Send a message to the AI service
-   */
-  public async sendMessage(cell_context?: string): Promise<void> {
-    return this.inputManager.sendMessage(cell_context);
-  }
-
-  /**
-   * Set the Anthropic prompt and tool blacklist for quick edit mode.
-   * This is used by the context cell highlighter for quick cell edits.
-   */
-  public setAnthropicPromptAndBlacklist(
-    prompt: string,
-    blacklist: string[]
-  ): void {
-    if (this.chatService instanceof AnthropicService) {
-      (this.chatService as AnthropicService).setSystemPrompt(prompt);
-      (this.chatService as AnthropicService).setToolBlacklist(blacklist);
-    }
-  }
-
   protected onAfterShow(): void {
     this.inputManager.focus();
   }
@@ -1195,12 +1344,12 @@ export class ChatBoxWidget extends Widget {
   /**
    * Show new chat display or history based on current thread state
    */
-  public showNewChatDisplayOrHistory(): void {
+  public async showNewChatDisplayOrHistory(): Promise<void> {
     const currentThread = this.chatHistoryManager.getCurrentThread();
     const hasMessages = currentThread && currentThread.messages.length > 0;
 
     if (hasMessages) {
-      this.showHistoryWidgetFromThread(currentThread);
+      await this.showHistoryWidgetFromThread(currentThread);
     } else {
       this.showNewChatDisplay();
     }
@@ -1252,8 +1401,8 @@ export class ChatBoxWidget extends Widget {
     this.showNewChatCta();
   }
 
-  public showHistoryWidgetFromThread(thread: IChatThread): void {
-    this.messageComponent.loadFromThread(thread);
+  public async showHistoryWidgetFromThread(thread: IChatThread): Promise<void> {
+    await this.threadManager.selectThread(thread.id);
     this.showHistoryWidget();
   }
 
@@ -1320,7 +1469,7 @@ export class ChatBoxWidget extends Widget {
         const newCurrentThread = this.chatHistoryManager.getCurrentThread();
         if (newCurrentThread) {
           this.threadNameDisplay.textContent = newCurrentThread.name;
-          this.messageComponent.loadFromThread(newCurrentThread);
+          await this.messageComponent.loadFromThread(newCurrentThread);
           if (newCurrentThread.messages.length > 0) {
             this.showHistoryWidget();
           } else {
@@ -1410,6 +1559,27 @@ export class ChatBoxWidget extends Widget {
     }
   }
 
+  /**
+   * Send a "Continue" message when the continue button is pressed
+   */
+  public sendContinueMessage(): void {
+    // Set the input value to "Continue"
+    this.inputManager.setInputValue('Continue');
+    
+    // Send the message
+    this.inputManager.sendMessage();
+    
+    // Hide the waiting reply box since user has responded
+    this.messageComponent.hideWaitingReplyBox();
+  }
+
+  /**
+   * Gets the message component for external access
+   */
+  public getMessageComponent(): ChatMessages {
+    return this.messageComponent;
+  }
+
   public dispose(): void {
     this.resizeObserver?.disconnect();
     this.mutationObserver?.disconnect();
