@@ -8,6 +8,9 @@ import { AppStateService } from '../AppState';
  */
 export class NotebookCellTools {
   private notebookTools: NotebookTools;
+  private isAutoScrollDisabled: boolean = false;
+  private lastAutoScrolledCell: HTMLElement | null = null;
+  private scrollListeners: Map<HTMLElement, () => void> = new Map();
 
   /**
    * Create a new NotebookCellTools instance
@@ -74,6 +77,8 @@ export class NotebookCellTools {
     // Activate the cell to run
     this.notebookTools.activateCell(cell);
 
+    const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+
     // Run the cell - pass the actual cell to run instead of using _findCellById
     await NotebookActions.runCells(
       notebook,
@@ -81,6 +86,8 @@ export class NotebookCellTools {
       current.widget?.sessionContext
     );
 
+    restoreScroll();
+
     // Process outputs in a formatted way
     const outputs: (string | { error: true; errorText: string })[] = [];
     try {
@@ -139,14 +146,17 @@ export class NotebookCellTools {
       outputs.push(`Error collecting outputs: ${error}`);
     }
 
+    // Handle smart scrolling after cell execution - scroll to include output
+    this.handleSmartScroll(cell.node, true);
+
     return outputs;
   }
 
   /**
-   * Adds a new cell (code, markdown, or raw) to the specified notebook at an optional position, returning its unique ID.
+   * Adds a new cell (code or markdown) to the specified notebook at an optional position, returning its unique ID.
    *
    * @param options Configuration options
-   * @param options.cell_type Type of cell ("code", "markdown", or "raw").
+   * @param options.cell_type Type of cell ("code" or "markdown").
    * @param options.source Initial source content for the cell.
    * @param options.summary A summary of the cell's content.
    * @param options.notebook_path Path to the notebook file (optional).
@@ -174,6 +184,10 @@ export class NotebookCellTools {
       );
     }
 
+    console.log(
+      `Adding cell of type ${options.cell_type} at position ${options.position} in notebook ${options.notebook_path || 'current'}`
+    );
+
     // Normalize the content - ensure we have valid strings
     const content = options.source || '';
 
@@ -187,6 +201,8 @@ export class NotebookCellTools {
       position = notebook.widgets.length; // Default to end of notebook
     }
 
+    const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+
     // Handle insertion based on position
     if (position === 0) {
       // Insert at the beginning of notebook
@@ -199,6 +215,8 @@ export class NotebookCellTools {
       NotebookActions.insertBelow(notebook);
     }
 
+    restoreScroll();
+
     let newCell = notebook.activeCell;
     if (!newCell) {
       throw new Error('Failed to create new cell');
@@ -213,7 +231,7 @@ export class NotebookCellTools {
 
     newCell = notebook.activeCell;
     if (!newCell) {
-      throw new Error('Failed to create new cell');
+      throw new Error('Failed to set active cell after insertion');
     }
 
     // Add tracking metadata to cell
@@ -246,6 +264,9 @@ export class NotebookCellTools {
     const trackingMetadata: any =
       newCell.model.sharedModel.getMetadata().cell_tracker;
 
+    // Handle smart scrolling after adding the cell - scroll to cell content only
+    this.handleSmartScroll(newCell.node);
+
     return trackingMetadata.trackingId;
   }
 
@@ -260,8 +281,8 @@ export class NotebookCellTools {
   /**
    * Ensure the first cell of a notebook is a plan cell
    */
-  public setFirstCellAsPlan(notebookPath?: string | null): void {
-    let current = this.notebookTools.getCurrentNotebook(notebookPath);
+  public setFirstCellAsPlan(notebookId?: string | null): void {
+    let current = this.notebookTools.getCurrentNotebook(notebookId);
     if (!current) return;
 
     let { notebook } = current;
@@ -277,7 +298,7 @@ export class NotebookCellTools {
 
     NotebookActions.changeCellType(notebook, 'markdown');
 
-    current = this.notebookTools.getCurrentNotebook(notebookPath);
+    current = this.notebookTools.getCurrentNotebook(notebookId);
     if (current && current.notebook) notebook = current.notebook;
     firstCell = notebook.widgets[0]; // Re-fetch after changing type
 
@@ -408,6 +429,9 @@ export class NotebookCellTools {
       }
     }
 
+    // Handle smart scrolling after removing cells - no specific cell to scroll to, skip
+    // Auto-scroll behavior will be handled when new cells are added/edited
+
     return cellsRemoved > 0;
   }
 
@@ -520,6 +544,9 @@ export class NotebookCellTools {
       cell.model.sharedModel.setSource(options.new_source);
     }
 
+    // Handle smart scrolling after editing the cell - scroll to cell content only
+    this.handleSmartScroll(cell.node);
+
     return true;
   }
 
@@ -549,29 +576,20 @@ export class NotebookCellTools {
     if (!planCell) {
       console.error('No first cell found for plan update');
       notebook.activeCellIndex = 0;
+      const restoreScroll = NotebookCellTools.preventScrolling(notebook);
       NotebookActions.insertAbove(notebook);
+      restoreScroll();
       notebook.activeCellIndex = 0;
       this.setFirstCellAsPlan(options.notebook_path);
     }
     const firstCell = notebook.widgets[0];
 
-    const oldContent = firstCell.model.sharedModel.getSource();
     const newContent = options.partial_plan;
 
-    // For edit_cell, we need to properly handle the diff calculation
-    // The streaming code might be partial, so we need to preserve the rest of the original content
-    let finalContent = oldContent;
+    firstCell.model.sharedModel.setSource(newContent);
 
-    if (oldContent && oldContent.length < newContent.length) {
-      // If the streaming code is shorter than original, preserve the remaining content
-      finalContent = oldContent + newContent.substring(oldContent.length);
-    } else if (oldContent && oldContent.length > newContent.length) {
-      // If streaming code is longer than original, use the new code as-is
-      finalContent = newContent;
-    }
-    // If lengths are equal, use the new code as-is
-
-    firstCell.model.sharedModel.setSource(finalContent);
+    // Handle smart scrolling after streaming plan update
+    // this.handleSmartScroll(notebook);
 
     return true;
   }
@@ -615,7 +633,9 @@ export class NotebookCellTools {
     if (!planCell) {
       console.error('No first cell found for plan update');
       notebook.activeCellIndex = 0;
+      const restoreScroll = NotebookCellTools.preventScrolling(notebook);
       NotebookActions.insertAbove(notebook);
+      restoreScroll();
       notebook.activeCellIndex = 0;
       this.setFirstCellAsPlan(options.notebook_path);
     }
@@ -665,16 +685,23 @@ export class NotebookCellTools {
     metadata.custom.next_step_string = options.next_step_string;
     firstCell.model.sharedModel.setMetadata(metadata);
 
+    const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+
     // Set the new plan content
     firstCell.model.sharedModel.setSource(newContent);
     void NotebookActions.runCells(notebook, [firstCell]);
 
+    restoreScroll();
+
     void AppStateService.getPlanStateDisplay().updatePlan(
       options.current_step_string,
       options.next_step_string,
       options.updated_plan_string
     );
 
+    // Handle smart scrolling after plan update
+    // this.handleSmartScroll(notebook);
+
     console.log('Plan cell updated successfully');
     return true;
   }
@@ -885,4 +912,254 @@ export class NotebookCellTools {
     console.log(result);
     return result;
   }
+
+  /**
+   * Scroll to show the cell content at the bottom of the viewport with smooth animation
+   * @param cellNode The HTML node of the cell to scroll to
+   * @param includeOutput Whether to include cell output in the scroll calculation (for run_cell)
+   */
+  private handleSmartScroll(
+    cellNode: HTMLElement,
+    includeOutput: boolean = false
+  ): void {
+    // Skip auto-scroll if disabled by user interaction
+    if (this.isAutoScrollDisabled) {
+      console.log('Auto-scroll disabled due to user interaction');
+      return;
+    }
+
+    // Find the scrollable container with the jp-WindowedPanel-outer class
+    const scrollContainer = cellNode.closest(
+      '.jp-WindowedPanel-outer'
+    ) as HTMLElement;
+
+    if (!scrollContainer) {
+      console.warn(
+        'Could not find scroll container with class jp-WindowedPanel-outer'
+      );
+      return;
+    }
+
+    // Get the cell input area (the actual cell content, not output)
+    const cellInputArea = cellNode.querySelector(
+      '.jp-InputArea'
+    ) as HTMLElement;
+    if (!cellInputArea) {
+      console.warn('Could not find cell input area');
+      return;
+    }
+
+    let targetElement = cellInputArea;
+
+    // For run_cell operations, include the output area if it exists
+    if (includeOutput) {
+      const outputArea = cellNode.querySelector(
+        '.jp-OutputArea'
+      ) as HTMLElement;
+      if (outputArea && outputArea.children.length > 0) {
+        targetElement = outputArea;
+      }
+    }
+
+    // Set up scroll listener for this container if not already set
+    this.setupScrollListener(scrollContainer, cellNode);
+
+    // Get the bottom position of the target element relative to the scroll container
+    const containerRect = scrollContainer.getBoundingClientRect();
+    const targetRect = targetElement.getBoundingClientRect();
+
+    // Calculate the scroll position to put the bottom of the target element
+    // at the bottom of the viewport with 20px padding
+    const targetBottom =
+      targetRect.bottom - containerRect.top + scrollContainer.scrollTop;
+    const desiredScrollTop = targetBottom - scrollContainer.clientHeight + 20;
+
+    // Get current scroll position
+    const currentScrollTop = scrollContainer.scrollTop;
+
+    // Only scroll if the desired position is below the current position (scroll down only)
+    // and if the user isn't already below the target area
+    if (desiredScrollTop <= currentScrollTop) {
+      console.log(
+        'Skipping scroll - target is above current position or user is already below target'
+      );
+      return;
+    }
+
+    // Store reference to this cell for scroll detection
+    this.lastAutoScrolledCell = cellNode;
+
+    // Smooth scroll to the calculated position (only scrolling down)
+    scrollContainer.scrollTo({
+      top: Math.max(0, desiredScrollTop),
+      behavior: 'smooth'
+    });
+
+    console.log(`Smart scrolled to cell content bottom with 20px padding`);
+  }
+
+  /**
+   * Set up scroll listener to detect user scrolling away from auto-scrolled cells
+   * @param scrollContainer The scroll container to monitor
+   * @param currentCell The current cell being auto-scrolled to
+   */
+  private setupScrollListener(
+    scrollContainer: HTMLElement,
+    currentCell: HTMLElement
+  ): void {
+    // Remove existing listener if any
+    const existingListener = this.scrollListeners.get(scrollContainer);
+    if (existingListener) {
+      scrollContainer.removeEventListener('scroll', existingListener);
+    }
+
+    let scrollTimeout: NodeJS.Timeout | null = null;
+    let lastScrollTime = Date.now();
+    let programmaticScroll = true; // Flag to ignore the initial programmatic scroll
+
+    const scrollListener = () => {
+      const now = Date.now();
+
+      // Ignore scroll events that happen immediately after we set programmaticScroll
+      // This gives time for our smooth scroll to complete
+      if (programmaticScroll && now - lastScrollTime < 1000) {
+        return;
+      }
+
+      programmaticScroll = false;
+
+      // Clear any existing timeout
+      if (scrollTimeout) {
+        clearTimeout(scrollTimeout);
+      }
+
+      // Debounce scroll events to avoid excessive checking
+      scrollTimeout = setTimeout(() => {
+        this.checkUserScrolledAway(scrollContainer, currentCell);
+      }, 100);
+    };
+
+    // Reset the flag after our programmatic scroll
+    setTimeout(() => {
+      programmaticScroll = false;
+      lastScrollTime = Date.now();
+    }, 100);
+
+    scrollContainer.addEventListener('scroll', scrollListener, {
+      passive: true
+    });
+    this.scrollListeners.set(scrollContainer, scrollListener);
+  }
+
+  /**
+   * Check if user has scrolled away from the bottom or back to the bottom of the notebook
+   * @param scrollContainer The scroll container
+   * @param currentCell The current cell we auto-scrolled to (not used in new logic)
+   */
+  private checkUserScrolledAway(
+    scrollContainer: HTMLElement,
+    currentCell: HTMLElement
+  ): void {
+    // Get the actual notebook instance
+    const current = this.notebookTools.getCurrentNotebook();
+    if (!current) return;
+
+    const { notebook } = current;
+
+    // Get the last cell from the notebook widgets
+    if (notebook.widgets.length === 0) return;
+
+    const lastCell = notebook.widgets[notebook.widgets.length - 1];
+    const lastCellNode = lastCell.node;
+    const lastCellInput = lastCellNode.querySelector(
+      '.jp-InputArea'
+    ) as HTMLElement;
+    if (!lastCellInput) return;
+
+    // Calculate scroll position information
+    const scrollTop = scrollContainer.scrollTop;
+    const scrollHeight = scrollContainer.scrollHeight;
+    const clientHeight = scrollContainer.clientHeight;
+
+    // Get container and last cell positions
+    const containerRect = scrollContainer.getBoundingClientRect();
+    const lastCellInputRect = lastCellInput.getBoundingClientRect();
+
+    // Calculate if we're near the bottom of the scrollable area (within 100px)
+    const distanceFromBottom = scrollHeight - (scrollTop + clientHeight);
+    const isNearScrollBottom = distanceFromBottom <= 10;
+
+    // Calculate if the last cell's input area is visible in the viewport
+    // The cell is considered visible if its bottom edge is within the viewport bounds (with buffer)
+    const lastCellInputBottom = lastCellInputRect.bottom;
+    const containerBottom = containerRect.bottom;
+    const isLastCellVisible = lastCellInputBottom <= containerBottom + 20; // 50px buffer
+
+    // User is considered "at the bottom" if either condition is true:
+    // 1. They're scrolled near the bottom of the document
+    // 2. The last cell's input area is visible in the viewport
+    const isAtBottom = isNearScrollBottom || isLastCellVisible;
+
+    // Update auto-scroll state based on position
+    if (isAtBottom && this.isAutoScrollDisabled) {
+      this.isAutoScrollDisabled = false;
+      console.log('User scrolled back to bottom, enabling auto-scroll', {
+        isNearScrollBottom,
+        isLastCellVisible,
+        distanceFromBottom
+      });
+    } else if (!isAtBottom && !this.isAutoScrollDisabled) {
+      this.isAutoScrollDisabled = true;
+      console.log('User scrolled away from bottom, disabling auto-scroll', {
+        isNearScrollBottom,
+        isLastCellVisible,
+        distanceFromBottom
+      });
+    }
+  }
+
+  /**
+   * Clean up scroll listeners
+   */
+  private cleanupScrollListeners(): void {
+    for (const [container, listener] of this.scrollListeners) {
+      container.removeEventListener('scroll', listener);
+    }
+    this.scrollListeners.clear();
+  }
+
+  /**
+   * Public cleanup method to be called when the instance is no longer needed
+   */
+  public dispose(): void {
+    this.cleanupScrollListeners();
+    this.isAutoScrollDisabled = false;
+    this.lastAutoScrolledCell = null;
+  }
+
+  /**
+   * Prevent scrolling to a cell or item in the notebook
+   * @param notebook The notebook to prevent scrolling from
+   * @returns A function to restore the default scrolling behavior
+   */
+  static preventScrolling(notebook: Notebook): () => void {
+    // Save the reference to the default scrollToCell method
+    const scrollToCellDefault = notebook.scrollToCell;
+    const scrollToItemDefault = notebook.scrollToItem;
+
+    notebook.scrollToCell = () =>
+      Promise.resolve(
+        console.warn('Scroll to cell prevented with intentional override')
+      );
+    notebook.scrollToItem = () =>
+      Promise.resolve(
+        console.warn('Scroll to item prevented with intentional override')
+      );
+
+    return () => {
+      // Restore the default scrollToCell method
+      notebook.scrollToCell = scrollToCellDefault;
+      notebook.scrollToItem = scrollToItemDefault;
+    };
+  }
 }
