@@ -25,7 +25,7 @@ export class NotebookDiffManager {
   private lastUserApprovalTime: number = 0; // Track when the user last approved changes
   private _shouldRunImmediately: boolean = false; // Track if we should run immediately
   private _shouldExecuteApprovedCells: boolean = false; // Track if we should execute all approved cells
-  private currentNotebookPath: string | null = null;
+  private currentNotebookId: string | null = null;
   private subscriptions: Subscription[] = [];
 
   // Signal that emits when diff processing is complete
@@ -50,13 +50,12 @@ export class NotebookDiffManager {
     this.diffApprovalDialog.setCallbacks({
       onApprove: trackingIds => this.approveDiffs(trackingIds),
       onReject: trackingIds => this.rejectDiffs(trackingIds),
-      onApproveAll: notebookPath => this.approveAllDiffs(notebookPath),
-      onRejectAll: notebookPath => this.rejectAllDiffs(notebookPath),
-      applyApprovedDiffs: notebookPath => this.applyApprovedDiffs(notebookPath),
-      handleRejectedDiffs: notebookPath =>
-        this.handleRejectedDiffs(notebookPath),
+      onApproveAll: notebookId => this.approveAllDiffs(notebookId),
+      onRejectAll: notebookId => this.rejectAllDiffs(notebookId),
+      applyApprovedDiffs: notebookId => this.applyApprovedDiffs(notebookId),
+      handleRejectedDiffs: notebookId => this.handleRejectedDiffs(notebookId),
       setExecuteApprovedCells: execute => this.setExecuteApprovedCells(execute),
-      onCellIdClick: cellId => this.notebookTools.scrollToCellById(cellId)
+      reapplyDiff: diffCell => this.reapplyDiff(diffCell)
     });
   }
 
@@ -67,9 +66,9 @@ export class NotebookDiffManager {
     // Subscribe to all diff state changes
     this.subscriptions.push(
       diffStateService.diffState$.subscribe(state => {
-        // Update current notebook path if it changes
-        if (state.notebookPath !== this.currentNotebookPath) {
-          this.currentNotebookPath = state.notebookPath || null;
+        // Update current notebook ID if it changes
+        if (state.notebookId !== this.currentNotebookId) {
+          this.currentNotebookId = state.notebookId || null;
         }
       })
     );
@@ -91,40 +90,40 @@ export class NotebookDiffManager {
   }
 
   /**
-   * Set the current notebook path context
-   * @param notebookPath Path to the notebook
+   * Set the current notebook ID context
+   * @param notebookId ID of the notebook
    */
-  public setNotebookPath(notebookPath: string | null): void {
-    if (this.currentNotebookPath === notebookPath) {
+  public setNotebookId(notebookId: string | null): void {
+    if (this.currentNotebookId === notebookId) {
       return; // No change needed
     }
 
     console.log(
-      `[NotebookDiffManager] Setting current notebook path: ${notebookPath}`
+      `[NotebookDiffManager] Setting current notebook ID: ${notebookId}`
     );
-    this.currentNotebookPath = notebookPath;
+    this.currentNotebookId = notebookId;
 
-    // Update DiffStateService with new notebook path
-    diffStateService.setNotebookPath(notebookPath);
-    this.diffApprovalDialog.updateNotebookPath(notebookPath || '');
+    // Update DiffStateService with new notebook ID (keeping path method for now)
+    diffStateService.setNotebookPath(notebookId);
+    this.diffApprovalDialog.updateNotebookPath(notebookId || '');
   }
 
   /**
    * Get diffs for a specific notebook by filtering the main collection
-   * @param notebookPath Path to the notebook
+   * @param notebookId Path to the notebook
    * @returns Array of diffs for the specified notebook path
    */
-  private getNotebookDiffs(notebookPath?: any): IPendingDiff[] {
+  private getNotebookDiffs(notebookId?: any): IPendingDiff[] {
     const currentState = diffStateService.getCurrentState();
 
     // If no path specified, return all diffs
-    if (!notebookPath) {
+    if (!notebookId) {
       return Array.from(currentState.pendingDiffs.values());
     }
 
     // Filter diffs by notebook path
     return Array.from(currentState.pendingDiffs.values()).filter(
-      (diff: IPendingDiff) => diff.notebookPath === notebookPath
+      (diff: IPendingDiff) => diff.notebookId === notebookId
     );
   }
 
@@ -140,8 +139,8 @@ export class NotebookDiffManager {
     for (const [, diff] of currentState.pendingDiffs) {
       // Only count diffs for current notebook
       if (
-        !this.currentNotebookPath ||
-        diff.notebookPath === this.currentNotebookPath
+        !this.currentNotebookId ||
+        diff.notebookId === this.currentNotebookId
       ) {
         total++;
         if (
@@ -174,16 +173,16 @@ export class NotebookDiffManager {
 
   /**
    * Track a cell addition diff and store its info
-   * @param notebookPath Optional path to the notebook
+   * @param notebookId Optional path to the notebook
    */
   public trackAddCell(
     trackingId: string,
     content: string,
     summary: string,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): void {
     // Use current notebook path if none specified
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     const pendingDiff = {
       cellId: trackingId,
@@ -191,7 +190,7 @@ export class NotebookDiffManager {
       newContent: content,
       updatedCellId: trackingId, // Initially the same as trackingId
       metadata: { summary },
-      notebookPath: path
+      notebookId: path
     };
 
     // Only add to DiffStateService - no local storage needed
@@ -204,17 +203,17 @@ export class NotebookDiffManager {
 
   /**
    * Track a cell edit diff and store its info
-   * @param notebookPath Optional path to the notebook
+   * @param notebookId Optional path to the notebook
    */
   public trackEditCell(
     trackingId: string,
     originalContent: string,
     newContent: string,
     summary: string,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): void {
     // Use current notebook path if none specified
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     const pendingDiff = {
       cellId: trackingId,
@@ -223,7 +222,7 @@ export class NotebookDiffManager {
       newContent,
       updatedCellId: trackingId, // Initially the same as trackingId
       metadata: { summary },
-      notebookPath: path
+      notebookId: path
     };
 
     // Only add to DiffStateService - no local storage needed
@@ -236,16 +235,16 @@ export class NotebookDiffManager {
 
   /**
    * Track a cell removal diff
-   * @param notebookPath Optional path to the notebook
+   * @param notebookId Optional path to the notebook
    */
   public trackRemoveCell(
     trackingId: string,
     originalContent: string,
     summary: string,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): void {
     // Use current notebook path if none specified
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     const pendingDiff = {
       cellId: trackingId,
@@ -253,7 +252,7 @@ export class NotebookDiffManager {
       originalContent,
       newContent: '', // Empty for remove
       metadata: { summary },
-      notebookPath: path
+      notebookId: path
     };
 
     // Only add to DiffStateService - no local storage needed
@@ -266,12 +265,12 @@ export class NotebookDiffManager {
 
   /**
    * Update the cell ID mapping when a cell ID changes due to diff display
-   * @param notebookPath Optional path to the notebook
+   * @param notebookId Optional path to the notebook
    */
   public updateCellIdMapping(
     originalCellId: string,
     updatedCellId: string,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): void {
     // Update the mapping
     this.cellIdMapping.set(originalCellId, updatedCellId);
@@ -307,7 +306,7 @@ export class NotebookDiffManager {
    * Check if there are any pending diffs
    */
   public hasPendingDiffs(): boolean {
-    return diffStateService.getPendingDiffCount(this.currentNotebookPath) > 0;
+    return diffStateService.getPendingDiffCount(this.currentNotebookId) > 0;
   }
 
   public hasRejectedDiffs(): boolean {
@@ -324,7 +323,7 @@ export class NotebookDiffManager {
    * Get the number of pending diffs
    */
   public getPendingDiffCount(): number {
-    return diffStateService.getPendingDiffCount(this.currentNotebookPath);
+    return diffStateService.getPendingDiffCount(this.currentNotebookId);
   }
 
   /**
@@ -332,25 +331,32 @@ export class NotebookDiffManager {
    * @param parentElement Parent element to attach the dialog to
    * @param useEmbeddedMode If true, will use embedded styling for chat context
    * @param isRunContext If true, indicates this approval is in the context of running code
-   * @param notebookPath Optional path to the notebook for filtering diffs
+   * @param notebookId Optional path to the notebook for filtering diffs
    * @returns Promise resolving to the approval status
    */
   public async showApprovalDialog(
     parentElement: HTMLElement,
     useEmbeddedMode: boolean = false,
     isRunContext: boolean = false,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): Promise<DiffApprovalStatus> {
     console.log(
       '[NotebookDiffManager] Showing approval dialog for current notebook'
     );
 
+    console.log(
+      `[NotebookDiffManager] Notebook ID: ${notebookId || this.currentNotebookId}`
+    );
+
     // Use provided notebook path or fall back to current notebook path
     const targetNotebookPath =
-      notebookPath !== undefined ? notebookPath : this.currentNotebookPath;
+      notebookId !== undefined ? notebookId : this.currentNotebookId;
 
     // Get diffs from DiffStateService for target notebook
     const diffs = this.getNotebookDiffs(targetNotebookPath);
+
+    console.log(diffs);
+
     if (diffs.length === 0) {
       console.log(
         '[NotebookDiffManager] No diffs to approve for target notebook'
@@ -396,15 +402,15 @@ export class NotebookDiffManager {
    * Show approval dialog specifically for cancellation scenario
    * This version shows the dialog with only approve/reject options (no run option)
    * @param parentElement Parent element for the dialog
-   * @param notebookPath Path to the notebook for filtering diffs
+   * @param notebookId Path to the notebook for filtering diffs
    * @returns Promise resolving to approval status
    */
   public async showCancellationApprovalDialog(
     parentElement: HTMLElement,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): Promise<DiffApprovalStatus> {
     // Use current notebook path if none specified
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     // Check if there are any diffs for this notebook path
     const diffs = this.getNotebookDiffs(path);
@@ -444,12 +450,10 @@ export class NotebookDiffManager {
 
   /**
    * Display diffs in their respective cells if they aren't already displayed
-   * @param notebookPath Optional path to the notebook containing the cells
+   * @param notebookId Optional path to the notebook containing the cells
    */
-  private async displayDiffsInCells(
-    notebookPath?: string | null
-  ): Promise<void> {
-    const path = notebookPath || this.currentNotebookPath;
+  private async displayDiffsInCells(notebookId?: string | null): Promise<void> {
+    const path = notebookId || this.currentNotebookId;
     const diffs = this.getNotebookDiffs(path);
 
     for (const diff of diffs) {
@@ -512,7 +516,7 @@ export class NotebookDiffManager {
       const diff = currentState.pendingDiffs.get(cellId);
       if (diff) {
         // Update DiffStateService
-        diffStateService.updateDiffState(cellId, true, diff.notebookPath);
+        diffStateService.updateDiffState(cellId, true, diff.notebookId);
       }
     }
   }
@@ -527,50 +531,82 @@ export class NotebookDiffManager {
       const diff = currentState.pendingDiffs.get(cellId);
       if (diff) {
         // Update DiffStateService
-        diffStateService.updateDiffState(cellId, false, diff.notebookPath);
+        diffStateService.updateDiffState(cellId, false, diff.notebookId);
       }
     }
   }
 
   /**
    * Approve all pending diffs
-   * @param notebookPath Optional path to the notebook to approve diffs for
+   * @param notebookId Optional path to the notebook to approve diffs for
    */
-  private approveAllDiffs(notebookPath: string | null = null): void {
+  private approveAllDiffs(notebookId: string | null = null): void {
     // Get diffs for the specified notebook or all diffs
-    const diffs = this.getNotebookDiffs(notebookPath);
+    const diffs = this.getNotebookDiffs(notebookId);
 
     // Set approved flag on all diffs
     for (const diff of diffs) {
       // Update DiffStateService
-      diffStateService.updateDiffState(diff.cellId, true, diff.notebookPath);
+      diffStateService.updateDiffState(diff.cellId, true, diff.notebookId);
     }
   }
 
   /**
    * Reject all pending diffs
-   * @param notebookPath Optional path to the notebook to reject diffs for
+   * @param notebookId Optional path to the notebook to reject diffs for
    */
-  public rejectAllDiffs(notebookPath: string | null = null): void {
+  public rejectAllDiffs(notebookId: string | null = null): void {
     // Get diffs for the specified notebook or all diffs
-    const diffs = this.getNotebookDiffs(notebookPath);
+    const diffs = this.getNotebookDiffs(notebookId);
 
     // Set rejected flag on all diffs
     for (const diff of diffs) {
+      if (diff.userDecision === 'approved' || diff.userDecision === 'run')
+        continue;
       // Update DiffStateService
-      diffStateService.updateDiffState(diff.cellId, false, diff.notebookPath);
+      diffStateService.updateDiffState(diff.cellId, false, diff.notebookId);
+    }
+  }
+
+  public rejectCellDiff(cellId: string): void {
+    const diff = diffStateService.getCurrentState().pendingDiffs.get(cellId);
+    if (diff) {
+      diffStateService.updateDiffState(cellId, false, diff.notebookId);
+      this.handleRejectedDiffs(diff.notebookId);
+    }
+  }
+
+  public reapplyDiff(diffCell: IPendingDiff): void {
+    if (diffCell.type !== 'edit') return;
+
+    diffStateService.addPendingDiff(diffCell.cellId, {
+      ...diffCell,
+      approved: undefined,
+      userDecision: undefined
+    });
+    this.displayDiffsInCells(diffCell.notebookId);
+    const llmStateDisplay =
+      AppStateService.getChatContainerSafe()?.chatWidget.llmStateDisplay;
+    if (llmStateDisplay) {
+      llmStateDisplay.showPendingDiffs(diffCell.notebookId, true);
+    }
+
+    // Also show diffs in DiffNavigationWidget for synchronized display
+    const diffNavigationWidget = AppStateService.getDiffNavigationWidgetSafe();
+    if (diffNavigationWidget) {
+      diffNavigationWidget.showPendingDiffs(diffCell.notebookId, true);
     }
   }
 
   /**
    * Apply all approved diffs
-   * @param notebookPath Optional path to the notebook containing the cells
+   * @param notebookId Optional path to the notebook containing the cells
    * @returns Promise resolving to a result object with success status and approval status
    */
   public async applyApprovedDiffs(
-    notebookPath?: string | null
+    notebookId?: string | null
   ): Promise<IDiffApplicationResult> {
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     // Get approved diffs from DiffStateService
     const approvedDiffs = this.getNotebookDiffs(path).filter(
@@ -662,13 +698,13 @@ export class NotebookDiffManager {
 
   /**
    * Handle rejected diffs (revert them)
-   * @param notebookPath Optional path to the notebook containing the cells
+   * @param notebookId Optional path to the notebook containing the cells
    * @returns Promise resolving to a result object with success status and approval status
    */
   public async handleRejectedDiffs(
-    notebookPath?: string | null
+    notebookId?: string | null
   ): Promise<IDiffApplicationResult> {
-    const path = notebookPath || this.currentNotebookPath;
+    const path = notebookId || this.currentNotebookId;
 
     // Get rejected diffs from DiffStateService
     const rejectedDiffs = this.getNotebookDiffs(path).filter(
@@ -727,7 +763,7 @@ export class NotebookDiffManager {
 
       // // // Clear the rejected diffs from DiffStateService
       // // for (const diff of rejectedDiffs) {
-      // //   diffStateService.removePendingDiff(diff.cellId, diff.notebookPath);
+      // //   diffStateService.removePendingDiff(diff.cellId, diff.notebookId);
       // // }
       //
       // // Clear mappings related to rejected diffs
@@ -751,12 +787,12 @@ export class NotebookDiffManager {
     }
   }
 
-  public async applyAllDiffs() {
-    const notebook = AppStateService.getState().currentNotebookPath;
-    await this.applyApprovedDiffs(notebook);
-    await this.handleRejectedDiffs(notebook);
-    this.clearDiffs();
-    this.diffApprovalDialog.close(this._shouldRunImmediately);
+  public approveCellDiff(cellId: string): void {
+    const diff = diffStateService.getCurrentState().pendingDiffs.get(cellId);
+    if (diff) {
+      diffStateService.updateDiffState(cellId, true, diff.notebookId);
+      this.applyApprovedDiffs(diff.notebookId);
+    }
   }
 
   /**
@@ -766,7 +802,7 @@ export class NotebookDiffManager {
     this.cellIdMapping.clear();
 
     // Clear from DiffStateService
-    diffStateService.clearAllDiffs(this.currentNotebookPath);
+    diffStateService.clearAllDiffs(this.currentNotebookId);
 
     console.log('[NotebookDiffManager] All diffs cleared');
   }
@@ -790,24 +826,13 @@ export class NotebookDiffManager {
     return should;
   }
 
-  /**
-   * Check if we should execute all approved cells after approval
-   * This flag is set when "Approve All and Run" is clicked
-   * and should be consumed after checking once
-   */
-  public shouldExecuteApprovedCells(): boolean {
-    const should = this._shouldExecuteApprovedCells;
-    this._shouldExecuteApprovedCells = false; // Consume the flag
-    return should;
-  }
-
   /**
    * Get all approved cell IDs (tracking IDs)
-   * @param notebookPath Optional path to filter by notebook
+   * @param notebookId Optional path to filter by notebook
    * @returns Array of approved cell tracking IDs
    */
-  public getApprovedCellIds(notebookPath?: string | null): string[] {
-    const path = notebookPath || this.currentNotebookPath;
+  public getApprovedCellIds(notebookId?: string | null): string[] {
+    const path = notebookId || this.currentNotebookId;
     const approvedCells = this.getNotebookDiffs(path).filter(
       diff =>
         diff.userDecision &&
@@ -849,20 +874,20 @@ export class NotebookDiffManager {
    * @param parentElement Parent element to attach the dialog to
    * @param useEmbeddedMode If true, will use embedded styling for chat context
    * @param isRunContext If true, indicates this approval is in the context of running code
-   * @param notebookPath Optional path to the notebook for filtering diffs
+   * @param notebookId Optional path to the notebook for filtering diffs
    */
   public startDiffApprovalProcess(
     parentElement: HTMLElement,
     useEmbeddedMode: boolean = false,
     isRunContext: boolean = false,
-    notebookPath?: string | null
+    notebookId?: string | null
   ): void {
     // Start the approval process without waiting
     this.showApprovalDialog(
       parentElement,
       useEmbeddedMode,
       isRunContext,
-      notebookPath
+      notebookId
     ).catch(error => {
       console.error('[NotebookDiffManager] Error during diff approval:', error);
       // Emit a rejection status on error
