@@ -25,6 +25,9 @@ import * as JsDiff from 'diff';
 import { ColorSchemeType } from 'diff2html/lib/types';
 import { timeout } from '../utils';
 import { Diff2HtmlUI } from 'diff2html/lib-esm/ui/js/diff2html-ui';
+import { NotebookCellTools } from './NotebookCellTools';
+import { inlineDiffService, MergeCallbacks } from './InlineDiffService';
+import { AppStateService } from '../AppState';
 
 export class NotebookDiffTools {
   private static themeObserver: MutationObserver | null = null;
@@ -131,48 +134,6 @@ export class NotebookDiffTools {
     return NotebookDiffTools.detectJupyterLabTheme() === ColorSchemeType.DARK;
   }
 
-  /**
-   * Refresh all visible diff overlays with the current theme
-   * This can be called manually or is automatically called on theme changes
-   */
-  public static refreshAllDiffDisplays(): void {
-    try {
-      const diffOverlays = document.querySelectorAll('.jp-DiffOverlay');
-
-      diffOverlays.forEach(overlay => {
-        const overlayElement = overlay as HTMLElement;
-        const cellId = overlayElement.dataset.cellId;
-        const diffType = overlayElement.dataset.diffType;
-
-        if (cellId && diffType) {
-          // Get the parent cell element
-          const cellElement = overlayElement.closest('.jp-Cell');
-          if (cellElement) {
-            // Find the metadata to regenerate the diff
-            const cell = cellElement as any; // Type assertion for accessing custom properties
-            if (cell._sage_diff_metadata) {
-              const { oldText, newText } = cell._sage_diff_metadata;
-
-              // Regenerate HTML with current theme
-              const newHtmlDiff = NotebookDiffTools.generateHtmlDiff(
-                oldText,
-                newText,
-                true
-              );
-              overlayElement.innerHTML = newHtmlDiff;
-
-              console.log(
-                `Refreshed diff display for cell ${cellId} with new theme`
-              );
-            }
-          }
-        }
-      });
-    } catch (error) {
-      console.error('Error refreshing diff displays:', error);
-    }
-  }
-
   /**
    * Calculate the difference between two strings line by line
    * @param oldText The original text
@@ -298,13 +259,13 @@ export class NotebookDiffTools {
   }
 
   /**
-   * Display a diff in a cell
+   * Display a diff in a cell using the InlineDiffService
    * @param notebookTools The notebook tools instance
    * @param cell The cell to display the diff in
    * @param oldText The original text content
    * @param newText The new text content
    * @param operation The operation being performed (add, edit, remove)
-   * @returns Object containing the HTML diff content and the updated cell info
+   * @returns Object containing the updated cell info (htmlDiff is empty string for compatibility)
    */
   display_diff(
     notebookTools: NotebookTools,
@@ -314,9 +275,6 @@ export class NotebookDiffTools {
     operation: string
   ): { htmlDiff: string; cell: Cell; cellId: string } {
     try {
-      // Initialize theme detection on first use
-      NotebookDiffTools.initializeThemeDetection();
-
       const normalizedOld = notebookTools.normalizeContent(oldText);
       const normalizedNew = notebookTools.normalizeContent(newText);
 
@@ -338,73 +296,41 @@ export class NotebookDiffTools {
       const updatedCell = notebookTools.getCurrentNotebook()?.notebook
         .activeCell as Cell;
 
-      // Generate HTML diff with diff2html
-      const htmlDiff = NotebookDiffTools.generateHtmlDiff(
+      // Set up merge callbacks to handle accept/reject operations
+      const mergeCallbacks: MergeCallbacks = {
+        onApproveAll: (cellId: string) => {
+          console.log(
+            `[NotebookDiffTools] Diff all resolved for cell ${cellId}`
+          );
+          AppStateService.getNotebookDiffManager().approveCellDiff(cellId);
+          this.cleanupDiffMetadata(updatedCell);
+        },
+        onAllResolved: (cellId: string) => {
+          console.log(
+            `[NotebookDiffTools] Diff all resolved for cell ${cellId}`
+          );
+          AppStateService.getNotebookDiffManager().approveCellDiff(cellId);
+          this.cleanupDiffMetadata(updatedCell);
+        },
+        onRejectAll: (cellId: string) => {
+          console.log(
+            `[NotebookDiffTools] Diff all resolved for cell ${cellId}`
+          );
+          AppStateService.getNotebookDiffManager().rejectCellDiff(cellId);
+          this.cleanupDiffMetadata(updatedCell);
+        }
+      };
+
+      // Use InlineDiffService to show the unified merge view
+      inlineDiffService.showInlineDiff(
+        updatedCell,
         normalizedOld,
         normalizedNew,
-        true
+        mergeCallbacks
       );
 
-      let host = updatedCell.node.getElementsByClassName('jp-InputArea-editor');
-
-      // Add back the setTimeout to ensure DOM is ready
-      setTimeout(async () => {
-        const item = host.item(0) as HTMLElement;
-
-        if (item) {
-          // Create overlay container
-          const overlayElement = document.createElement('div');
-          overlayElement.className = 'jp-DiffOverlay';
-          overlayElement.style.position = 'absolute';
-          overlayElement.style.top = '0';
-          overlayElement.style.left = '0';
-          overlayElement.style.width = '100%';
-          overlayElement.style.height = '100%';
-          overlayElement.style.zIndex = '100';
-          // overlayElement.style.overflow = 'no';
-          overlayElement.style.backgroundColor = 'var(--jp-layout-color0)';
-          overlayElement.innerHTML = htmlDiff;
-
-          // Add data attributes to identify this overlay
-          overlayElement.dataset.cellId = updatedCell.model.id;
-          overlayElement.dataset.diffType = operation;
-
-          // Store diff metadata on the cell element for theme refreshing
-          const cellElement = updatedCell.node;
-          (cellElement as any)._sage_diff_metadata = {
-            oldText: normalizedOld,
-            newText: normalizedNew,
-            operation
-          };
-
-          // Check if item has position set
-          const itemPosition = window.getComputedStyle(item).position;
-          if (itemPosition === 'static') {
-            item.style.position = 'relative';
-          }
-
-          // Remove any existing overlays for this cell
-          const existingOverlay = item.querySelector('.jp-DiffOverlay');
-          if (existingOverlay) {
-            existingOverlay.remove();
-          }
-
-          // Append the overlay to the editor area
-          item.appendChild(overlayElement);
-
-          await timeout(50);
-
-          const height = item.querySelector('.d2h-file-wrapper')?.clientHeight;
-
-          console.log(height);
-          if (height) {
-            item.style.minHeight = `${height}px`;
-          }
-        }
-      }, 100); // 100ms delay to ensure DOM is ready
-
       return {
-        htmlDiff,
+        htmlDiff: '', // Empty string for compatibility - inline diff doesn't use HTML overlays
         cell: updatedCell,
         cellId: updatedCell.model.id
       };
@@ -437,6 +363,36 @@ export class NotebookDiffTools {
     }
   }
 
+  /**
+   * Clean up diff metadata from a cell
+   * @param cell The cell to clean up metadata from
+   */
+  private cleanupDiffMetadata(cell: Cell): void {
+    try {
+      const meta = cell.model.sharedModel.getMetadata() || {};
+      const cleanedMeta = { ...meta };
+
+      if (cleanedMeta.custom) {
+        // Fix the spread operator issue by checking if custom is an object
+        const customData: any =
+          typeof cleanedMeta.custom === 'object' && cleanedMeta.custom
+            ? { ...cleanedMeta.custom }
+            : {};
+
+        // Create a clean version without the diff property
+        if (customData) {
+          delete customData.diff;
+          cleanedMeta.custom = customData;
+        }
+      }
+
+      cell.model.sharedModel.setMetadata(cleanedMeta);
+      console.log('Cleaned up diff metadata for cell:', cell.model.id);
+    } catch (error) {
+      console.error('Error cleaning up diff metadata:', error);
+    }
+  }
+
   /**
    * Store original and new content in cell metadata
    */
@@ -478,7 +434,7 @@ export class NotebookDiffTools {
   }
 
   /**
-   * Apply or reject the diff stored in metadata
+   * Apply or reject the diff using InlineDiffService chunk acceptance
    */
   apply_diff(
     notebookTools: NotebookTools,
@@ -486,6 +442,9 @@ export class NotebookDiffTools {
     accept: boolean
   ): { success: boolean; updatedCellId?: string } {
     try {
+      console.log(
+        'APPLYING DIFF ================================== APPLYING DIFF ============== APPYING DIFF'
+      );
       const notebook = notebookTools.getCurrentNotebook()?.notebook;
 
       notebookTools.activateCell(cell);
@@ -493,7 +452,11 @@ export class NotebookDiffTools {
         .activeCell as Cell;
 
       if (cell.model.type === 'markdown' && notebook) {
+        const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+
         void NotebookActions.runCells(notebook, [cell]);
+
+        restoreScroll();
       }
 
       const meta = active.model.sharedModel.getMetadata() || {};
@@ -511,13 +474,14 @@ export class NotebookDiffTools {
         originalCellType = 'code'
       } = diffMeta;
 
-      // First, remove the diff overlay
-      NotebookDiffTools.removeDiffOverlay(active);
+      // Get the cell ID for the inline diff service
+      const cellId = (
+        active.model.sharedModel.getMetadata()?.cell_tracker as any
+      )?.trackingId;
 
-      // Apply or revert
       if (accept) {
-        if (operation !== 'remove') {
-          active.model.sharedModel.setSource(newContent);
+        if (operation !== 'remove' && cellId && inlineDiffService) {
+          inlineDiffService.acceptAllChunks(cellId);
         }
       } else {
         if (operation === 'add') {
@@ -525,6 +489,10 @@ export class NotebookDiffTools {
         } else {
           active.model.sharedModel.setSource(originalContent);
         }
+
+        if (cellId && inlineDiffService) {
+          inlineDiffService.cleanupMergeView(cellId);
+        }
       }
 
       // Restore cell type if changed
@@ -538,22 +506,8 @@ export class NotebookDiffTools {
         }
       }
 
-      // Remove diff metadata
-      const cleanedMeta = { ...meta };
-      if (cleanedMeta.custom) {
-        // Fix the spread operator issue by checking if custom is an object
-        const customData: any =
-          typeof cleanedMeta.custom === 'object' && cleanedMeta.custom
-            ? { ...cleanedMeta.custom }
-            : {};
-
-        // Create a clean version without the diff property
-        if (customData) {
-          delete customData.diff;
-          cleanedMeta.custom = customData;
-        }
-      }
-      active.model.sharedModel.setMetadata(cleanedMeta);
+      // Clean up diff metadata using the helper method
+      this.cleanupDiffMetadata(active);
 
       console.log(
         'Cell metadata after applying diff:',
@@ -567,6 +521,34 @@ export class NotebookDiffTools {
     }
   }
 
+  /**
+   * Accept all remaining chunks in a cell's merge view programmatically
+   * @param cell The cell to accept all chunks for
+   * @returns Promise<boolean> indicating success
+   */
+  async acceptAllChunks(cell: Cell): Promise<boolean> {
+    const cellId = (cell.model.sharedModel.getMetadata()?.cell_tracker as any)
+      ?.trackingId;
+    if (!cellId || !inlineDiffService) return false;
+
+    await inlineDiffService.acceptAllChunks(cellId);
+    return true;
+  }
+
+  /**
+   * Reject all remaining chunks in a cell's merge view programmatically
+   * @param cell The cell to reject all chunks for
+   * @returns Promise<boolean> indicating success
+   */
+  async rejectAllChunks(cell: Cell): Promise<boolean> {
+    const cellId = (cell.model.sharedModel.getMetadata()?.cell_tracker as any)
+      ?.trackingId;
+    if (!cellId || !inlineDiffService) return false;
+
+    await inlineDiffService.rejectAllChunks(cellId);
+    return true;
+  }
+
   /**
    * Log diff to console, now using diff2html for HTML portion
    */
