@@ -10,6 +10,9 @@ import { TrackingIDUtility } from '../TrackingIDUtility';
 import { WaitingUserReplyBoxManager } from './WaitingUserReplyBoxManager';
 import { FilesystemTools } from '../BackendTools/FilesystemTools';
 import { WebTools } from '../BackendTools/WebTools';
+import { AppStateService } from '../AppState';
+import { Subscription } from 'rxjs';
+
 /**
  * Class providing tools for manipulating notebook cells
  */
@@ -20,6 +23,7 @@ export class NotebookTools {
   private waitingUserReplyBoxManager: WaitingUserReplyBoxManager;
   private filesystemTools: FilesystemTools;
   private webTools: WebTools;
+  private subscriptions: Subscription[] = [];
   /**
    * Create a new NotebookTools instance
    * @param notebooks The notebook tracker from JupyterLab
@@ -34,45 +38,25 @@ export class NotebookTools {
     this.filesystemTools = new FilesystemTools();
     this.webTools = new WebTools();
     this.cellTools = new NotebookCellTools(this);
-  }
 
-  /**
-   * Find a notebook by its path
-   * @param notebookPath Path to the notebook
-   * @returns The notebook widget or null if not found
-   */
-  public findNotebookByPath(
-    notebookPath?: string | null
-  ): { notebook: Notebook; widget: any } | null {
-    if (!notebookPath) {
-      return null;
-    }
-
-    let foundNotebook: { notebook: Notebook; widget: any } | null = null;
-
-    this.notebookTracker.forEach(widget => {
-      if (widget.context.path === notebookPath) {
-        foundNotebook = { notebook: widget.content, widget: widget };
-      }
-    });
-
-    return foundNotebook;
+    // Subscribe to notebook change events (no need to track locally)
   }
 
   /**
    * Get the current active notebook or return null if none
-   * @param notebookPath Optional notebook path to specify which notebook to get
+   * @param notebookIdentifier Optional notebook path or ID to specify which notebook to get
    * @returns The current notebook or null
    */
   public getCurrentNotebook(
-    notebookPath?: string | null
+    notebookIdentifier?: string | null
   ): { notebook: Notebook; widget: NotebookPanel } | null {
-    // If a specific notebook path is provided, find that notebook
-    if (notebookPath) {
-      return this.findNotebookByPath(notebookPath);
+    // First try to get the notebook from AppStateService
+    const currentNotebook = AppStateService.getCurrentNotebook();
+    if (currentNotebook) {
+      return { notebook: currentNotebook.content, widget: currentNotebook };
     }
 
-    // Otherwise use the current active notebook
+    // Otherwise fallback to the current active notebook from tracker
     const current = this.notebookTracker.currentWidget;
 
     if (!current) {
@@ -169,8 +153,11 @@ export class NotebookTools {
     }
   }
 
-  public async getNotebookSummary(notebookPath: string | null): Promise<any> {
-    const nb = this.getCurrentNotebook(notebookPath);
+  public async getNotebookSummary(
+    notebookIdentifier: string | null
+  ): Promise<any> {
+    // Support both notebook paths and IDs for backward compatibility
+    const nb = this.getCurrentNotebook(notebookIdentifier);
     const cells = [];
     let index = 0;
     for (const cell of nb?.widget.model?.cells || []) {
@@ -519,10 +506,26 @@ export class NotebookTools {
    */
   wait_user_reply(options: { notebook_path?: string | null }): boolean {
     try {
-      // Show the waiting reply box
-      this.waitingUserReplyBoxManager.show();
-      console.log('Waiting for user reply - box shown');
-      return true;
+      // Get the ChatMessages component from the current chat container
+      const chatContainer = AppStateService.getState().chatContainer;
+      if (chatContainer && chatContainer.chatWidget) {
+        const messageComponent = chatContainer.chatWidget.getMessageComponent();
+        
+        // Set the continue callback if not already set
+        messageComponent.setContinueCallback(() => {
+          chatContainer.chatWidget.sendContinueMessage();
+        });
+        
+        // Show the waiting reply box using the new system
+        messageComponent.showWaitingReplyBox();
+        console.log('Waiting for user reply - new system box shown');
+        return true;
+      } else {
+        // Fallback to old system if chat container is not available
+        this.waitingUserReplyBoxManager.show();
+        console.log('Waiting for user reply - fallback to old system');
+        return true;
+      }
     } catch (error) {
       console.error('Error showing waiting reply box:', error);
       return false;
