@@ -2,6 +2,7 @@ import { ChatHistoryManager, IChatThread } from './Chat/ChatHistoryManager';
 import { ChatMessages } from './Chat/ChatMessages';
 import { IChatService } from './Services/IChatService';
 import { AppStateService } from './AppState';
+import { StateDBCachingService } from './utils/stateDBCaching';
 
 /**
  * Manages thread operations for the chatbox
@@ -12,7 +13,7 @@ export class ThreadManager {
   private chatService: IChatService;
   private leftSideBanner!: HTMLDivElement;
   private bannerOverlay!: HTMLDivElement;
-  private currentNotebookPath: string | null = null;
+  private currentNotebookId: string | null = null;
   private chatNode: HTMLElement;
   private threadNameDisplay: HTMLElement;
 
@@ -35,28 +36,138 @@ export class ThreadManager {
     // Initialize the banner
     this.initializeBanner();
 
+    // Subscribe to notebook change events from AppStateService
+    AppStateService.onNotebookChanged().subscribe(({ newNotebookId }) => {
+      if (newNotebookId) {
+        this.setNotebookId(newNotebookId);
+      }
+    });
+
     // Update the thread name display
     this.updateThreadSelectorButtonText(
       this.chatHistoryManager.getCurrentThread()?.name || 'New Chat'
     );
   }
 
-  public updateNotebookPath(newPath: string): void {
-    this.currentNotebookPath = newPath;
+  public updateNotebookId(newId: string): void {
+    this.setNotebookId(newId);
   }
 
   /**
-   * Set the current notebook path
-   * @param notebookPath Path to the notebook
+   * Set the current notebook ID
+   * @param notebookId ID of the notebook
    */
-  public setNotebookPath(notebookPath: string | null): void {
-    this.currentNotebookPath = notebookPath;
+  public setNotebookId(notebookId: string | null): void {
+    this.currentNotebookId = notebookId;
     this.refreshBannerIfVisible();
     this.updateThreadSelectorButtonText(
       this.chatHistoryManager.getCurrentThread()?.name || 'New Chat'
     );
   }
 
+  /**
+   * Store the last selected thread for a notebook
+   * @param notebookId ID of the notebook
+   * @param threadId ID of the thread to remember
+   */
+  public async storeLastThreadForNotebook(
+    notebookId: string,
+    threadId?: string
+  ): Promise<void> {
+    try {
+      const key = `last-thread-${notebookId}`;
+      if (threadId) await StateDBCachingService.setValue(key, threadId);
+      else await StateDBCachingService.removeValue(key);
+      console.log(
+        `[ThreadManager] Stored last thread ${threadId} for notebook ${notebookId}`
+      );
+    } catch (error) {
+      console.warn(`[ThreadManager] Failed to store last thread:`, error);
+    }
+  }
+
+  /**
+   * Get the last selected thread for a notebook
+   * @param notebookId ID of the notebook
+   * @returns The thread ID if found, null otherwise
+   */
+  public async getLastThreadForNotebook(
+    notebookId: string
+  ): Promise<string | null> {
+    try {
+      const key = `last-thread-${notebookId}`;
+      const threadId = await StateDBCachingService.getValue(key, null);
+      console.log(
+        `[ThreadManager] Retrieved last thread ${threadId} for notebook ${notebookId}`
+      );
+      return threadId;
+    } catch (error) {
+      console.warn(`[ThreadManager] Failed to get last thread:`, error);
+      return null;
+    }
+  }
+
+  /**
+   * Get the last selected thread object for a notebook, or null if not found
+   * @param notebookId ID of the notebook
+   * @returns The thread object if found and valid, null otherwise
+   */
+  public async getLastValidThreadForNotebook(
+    notebookId: string
+  ): Promise<IChatThread | null> {
+    try {
+      const threadId = await this.getLastThreadForNotebook(notebookId);
+      if (!threadId) {
+        return null;
+      }
+
+      // Check if the thread still exists in the chat history
+      const threads = this.chatHistoryManager.getThreadsForNotebook(notebookId);
+      if (!threads) {
+        console.log(
+          `[ThreadManager] No threads found for notebook ${notebookId}`
+        );
+        await this.clearLastThreadForNotebook(notebookId);
+        return null;
+      }
+
+      const thread = threads.find(t => t.id === threadId);
+
+      if (thread) {
+        console.log(
+          `[ThreadManager] Found valid last thread: ${thread.name} for notebook ${notebookId}`
+        );
+        return thread;
+      } else {
+        console.log(
+          `[ThreadManager] Last thread ${threadId} no longer exists for notebook ${notebookId}`
+        );
+        // Clean up invalid reference
+        await this.clearLastThreadForNotebook(notebookId);
+        return null;
+      }
+    } catch (error) {
+      console.warn(`[ThreadManager] Failed to get last valid thread:`, error);
+      return null;
+    }
+  }
+
+  /**
+   * Clear the stored last thread for a notebook
+   * @param notebookId ID of the notebook
+   */
+  public async clearLastThreadForNotebook(notebookId: string): Promise<void> {
+    try {
+      const key = `last-thread-${notebookId}`;
+      await StateDBCachingService.removeValue(key);
+      console.log(
+        `[ThreadManager] Cleared last thread for notebook ${notebookId}`
+      );
+    } catch (error) {
+      console.warn(`[ThreadManager] Failed to clear last thread:`, error);
+    }
+  }
+
   /**
    * Filter thread list to display only one "New Chat" (the most recent)
    * @param threads Sorted threads (most recent first)
@@ -95,14 +206,19 @@ export class ThreadManager {
    * Select a specific thread and load its history
    * @param threadId ID of the thread to select
    */
-  public selectThread(threadId: string): void {
+  public async selectThread(threadId: string): Promise<void> {
     // First, cancel any ongoing request in the chat service
     this.chatService.cancelRequest();
+    let thread = this.chatHistoryManager.getCurrentThread();
+    if (threadId !== this.chatHistoryManager.getCurrentThread()?.id) {
+      thread = this.chatHistoryManager.switchToThread(threadId);
+      const nbId = AppStateService.getState().currentNotebookId;
+      if (nbId) await this.storeLastThreadForNotebook(nbId, thread?.id);
+    }
 
-    const thread = this.chatHistoryManager.switchToThread(threadId);
     if (thread) {
       // Load the selected thread
-      this.messageComponent.loadFromThread(thread);
+      await this.messageComponent.loadFromThread(thread);
 
       // Reset the conversation history in chatService
       this.resetChatServiceHistory();
@@ -123,9 +239,9 @@ export class ThreadManager {
   /**
    * Create a new chat thread
    */
-  public createNewThread(): IChatThread | null {
+  public async createNewThread(): Promise<IChatThread | null> {
     // Only proceed if we have an active notebook
-    if (!this.currentNotebookPath) {
+    if (!this.currentNotebookId) {
       return null;
     }
 
@@ -150,8 +266,11 @@ export class ThreadManager {
     const newThread = this.chatHistoryManager.createNewThread('New Chat');
 
     if (newThread) {
+      const nbId = AppStateService.getState().currentNotebookId;
+      if (nbId) await this.storeLastThreadForNotebook(nbId, newThread?.id);
+
       // Load the empty thread into the UI
-      this.messageComponent.loadFromThread(newThread);
+      await this.messageComponent.loadFromThread(newThread);
 
       // Reset the chat service history
       this.chatService.resetConversationHistory();
@@ -346,7 +465,7 @@ export class ThreadManager {
     // Clear existing content
     threadsContainer.innerHTML = '';
 
-    if (!this.currentNotebookPath) {
+    if (!this.currentNotebookId) {
       threadsContainer.innerHTML =
         '<div class="sage-ai-banner-empty">No notebook selected</div>';
       return;
@@ -382,8 +501,8 @@ export class ThreadManager {
         <div class="sage-ai-banner-thread-date">${formatDate(thread.lastUpdated)}</div>
       `;
 
-      threadItem.addEventListener('click', () => {
-        this.selectThread(thread.id);
+      threadItem.addEventListener('click', async () => {
+        await this.selectThread(thread.id);
         this.closeBanner();
       });
 
